#include <sdsl/algorithms.hpp>
#include <sdsl/util.hpp>
#include <sdsl/suffixarrays.hpp>
#include <sdsl/suffixtrees.hpp>
#include <iostream>

using namespace sdsl;
using namespace std;

template<class Csa>
void csa_info(const Csa &csa){
	cout << "alphabet size      : " << (int)csa.sigma << endl;
	cout << "text size          : " << csa.size() << endl;
	cout << "index size in MB   : " << ((double)util::get_size_in_bytes(csa))/(1024.0*1024.0) << endl;
}

template<class Csa>
void test_csa(const Csa &csa, const unsigned char* text){
	typedef typename Csa::size_type size_type;
	for(size_type i=0; i < csa.size(); ++i){
		for(size_type l=1; l < csa.size()-i; ++l){
			size_type r = algorithm::count(csa, text+i, l);
			cout << i << " " << l << " " << r << endl; 
		}
	}
	if( algorithm::count(csa, (const unsigned char*)"xxx", 1) != 0 ){
		cout << "ERROR found x in the text!!!" << endl;
	}
}

int main(){
	const unsigned char *text = (const unsigned char*)"el_anele_lepanelen";
	typedef csa_wt<>::size_type size_type;
	csa_wt<> wt_index(text);
	csa_info(wt_index);
	cout << algorithm::extract(wt_index, 0, wt_index.size()) << endl;
	test_csa(wt_index, text);

	unsigned char *text2 = new unsigned char[wt_index.size()];
	algorithm::extract(wt_index, 0, wt_index.size()-1, text2);
	cout << text2 << endl;
	cout << algorithm::extract(wt_index, 0, wt_index.size()-1) << endl;

	cst_sct2<> t_index(text);
	for(size_type i=0; i< t_index.lcp.size(); ++i)
		cout << t_index.lcp[i] << " ";
	cout << endl;
	cout << t_index.bp << endl;
	cout << t_index.first_child_bv << endl;
	
	unsigned char *pat = (unsigned char*)"len";
	string str;
	while(cin>>str){
		/*
		cst_sct<>::node_type node = t_index.root();
		cst_sct<>::size_type char_pos = 0;
		cout <<
		algorithm::forward_search(t_index, node, 0, (const unsigned char*)str.c_str(), str.size(), char_pos)
		<< endl;
		*/
		cout << algorithm::count(t_index, (const unsigned char*)str.c_str(), str.size()) << endl;
		std::vector<uint32_t> occs; 
		algorithm::locate(t_index, (const unsigned char*)str.c_str(), str.size(), occs);
		for(size_t i=0; i<occs.size(); ++i){
			cout<<" "<<occs[i]<<endl;
		}
		cout<< algorithm::extract(t_index, t_index.root()) << endl;
		cst_sct2<>::node_type no = t_index.child(t_index.root(), (unsigned char)'a');
		cout<< algorithm::extract(t_index, no) << endl;
		no = t_index.child(no, (unsigned char)'n');
		cout << no << endl;
		cout<< algorithm::extract(t_index, no) << endl;
		no = t_index.parent(no);
		cout << no << endl;
		cout<< algorithm::extract(t_index, no) << endl;
		no = t_index.parent(no);
		cout << no << endl;
		cout<< algorithm::extract(t_index, no) << endl;

		for(cst_sct2<>::const_iterator it=t_index.begin(); it!=t_index.end(); ++it){
			cout << *it << endl;
		}
	}
	delete [] text2;
}
