#include "sdsl/suffixtrees.hpp"
#include "sdsl/algorithms.hpp"
#include "sdsl/lcp_uncompressed.hpp"
#include "sdsl/util.hpp"
#include "sdsl/testutils.hpp"

#include <iostream>
#include <vector>
#include <list>
#include <string>
#include <string.h>
#include <utility>
#include <iomanip>
#include <sys/time.h>
#include <time.h>

using namespace sdsl;
using namespace std;

//typedef cst_sct<> tCST;
typedef csa_wt<csa_wt<>::wavelet_tree_type, 10000, 10000> tCSA_WT;
typedef cst_sct2<tCSA_WT, lcp_kurtz<tCSA_WT >/*lcp_uncompressed<32>*/> tCST;
typedef tCST::size_type size_type;
typedef vector<size_type> tVI;
typedef pair<size_type, size_type> tPII;
typedef vector<tPII> tVPII;

bool backward_search(
		const tCST::csa_type &csa,
		const unsigned char c,
		tCST::node_type &p){
	typedef tCST::size_type size_type;
	size_type cc = csa.char2comp[c];
	if (!cc)
		return false;
	/*
	if (p.left == p.right-1) {
		int c_before_l = csa.wavelet_tree.rank_special(p.left, c);
		if (c_before_l < 0)
			return false;
		p.left = csa.C[cc]+c_before_l;
		p.right = p.left + 1;
		return true;
	}*/
	size_type c_before_r = csa.wavelet_tree.rank(p.right, c);
	if (!c_before_r)
		return false;
	size_type c_before_l = csa.wavelet_tree.rank(p.left, c);
	if (c_before_l == c_before_r)
		return false;
	p.left = csa.C[cc]+c_before_l;
	p.right = csa.C[cc]+c_before_r;
	return true;
}

inline int getWeight(int parent_lcp, int length) {
//	return (length * (length + 1))/2 - (parent_lcp * (parent_lcp + 1))/2;
	return length - parent_lcp;
}

float computeK(const tCST &cst, float *valtab, const unsigned char *s, int length) {
	int l1, l2, ind;
	float res = 0;
	tCST::node_type p(0, 0, cst.size());
	// calculate matching statistics
	for (int p2 = length-1; p2 >= 0; --p2)
		if( backward_search(cst.csa, s[p2], p) ){
			++p.l;
			l1 = cst.lcp[p.left];
			l2 = (p.right < cst.size()? cst.lcp[p.right] : -1);
			if (l1 < l2) {
				ind = p.right;
				l1 = l2;
			}
			else
				ind = p.left;
			res += valtab[ind] + getWeight(l1, p.l) * (p.right-p.left);
		}else if (p.l > 0){
			--p.right;
			p = cst.parent(p);
			++p.right;
			++p2;
		}
	return res;
}

void compute_valtab(const tCST &cst1, int i, int j, int l, int idx, long long val, float *valtab) {
	int lb = i, m;
	do {
		valtab[idx] = val;
		if (lb < idx-1) {
			size_type t = cst1.bp_support.select(idx+1)-1;
			size_type k = cst1.bp_support.rank(cst1.bp_support.find_open(t))-1;
			m = cst1.lcp[k];
			compute_valtab(cst1, lb, idx-1, m, k, val + getWeight(l, m) * (idx - lb), valtab);
		}
		if (idx == j)
			return;
		lb = idx;
		size_type t = cst1.bp_support.find_close(cst1.bp_support.select(idx+1))-1;
		idx = cst1.bp_support.rank(cst1.bp_support.find_open(t)) - 1;
	} while((m = cst1.lcp[idx]) == l);
	compute_valtab(cst1, lb, j, m, idx, val + getWeight(l, m) * (j - lb + 1), valtab);
}

int main(int argc, char **argv){
	/*
	char S1[20]="acaaacatat";
	char S2[20]="caaca";
	tCST T1((const unsigned char *)S1);
	float *valtab2 = new float[T1.size()];
	compute_valtab(T1, 0, T1.size()-1, 0, 1, 0, valtab2);
	for (int i=0; i<=10; ++i)
		printf("%.0f ", valtab2[i]);
	puts("");
	printf("%.0f\n", computeK(T1, valtab2, (const unsigned char *)S2, 5));
	delete [] valtab2;
	return 0;
	*/
	if (argc < 5) {
		printf("usage: %s <textfile> <length of text file> <patternfile> <length of pattern file>\n", argv[0]);
		exit(1);
	}
	int length1, length2;
	sscanf(argv[2], "%d", &length1);
	sscanf(argv[4], "%d", &length2);

	//' text, pattern and log files
	ifstream text_fp(argv[1]);
	ifstream pattern_fp(argv[3]);

	if(!text_fp) {
		cout << "Error opening text file : " << argv[1] << endl;
		exit(1);
	}
	if(!pattern_fp) {
		cout << "Error opening pattern file : " << argv[3] << endl;
		exit(1);
	}
	float *valtab;

	char *text = new char[length1+1];
	text_fp.read(text,length1);
	text_fp.close();
	text[length1] = 0;

	stop_watch sw;
	sw.start();
	ifstream in("index");
	tCST cst1;
	if (!in) {
		cst1 = tCST((const unsigned char *)text);
//		util::store_to_file(cst1, "index");
	}
	/*
	else {
		util::load_from_file(cst1, "index");
	}o*/
	sw.stop();
	cout << "Construction: " << sw.getUserTime() << endl;
	/*
	cout << (int)cst1.csa.sigma << endl;
	for (int i=0; i<cst1.csa.sigma; ++i)
		cout << cst1.csa.comp2char[i];
	cout << endl;*/
	delete [] text;
	valtab = new float[cst1.size()];
	cout << util::get_size_in_bytes(cst1) + sizeof(float) * cst1.size() << endl;
	cout << util::get_size_in_bytes(cst1) << endl;
	cout << util::get_size_in_bytes(cst1.csa) << endl;
	cout << util::get_size_in_bytes(cst1.lcp) << endl;
	cout << util::get_size_in_bytes(cst1.bp) << endl;
	cout << util::get_size_in_bytes(cst1.bp_support) << endl;
	sw.start();
	compute_valtab(cst1, 0, cst1.size()-1, 0, 1, 0, valtab);
	sw.stop();
	cout << "precompute: " << sw.getUserTime() << endl;
	char *pattern = new char[length2 + 1];
	pattern_fp.read(pattern,length2);
	pattern_fp.close();
	sw.start();
	float res = computeK(cst1, valtab, (unsigned char *)pattern, length2);
	sw.stop();
	cout << "computeK: " << sw.getUserTime() << endl;
	cout << res << endl;
	delete [] valtab;
	return 0;
}
