#ifndef INCLUDED_SDSL_BALANCED_PARENTHESES_SUPPORT_G
#define INCLUDED_SDSL_BALANCED_PARENTHESES_SUPPORT_G

#include "int_vector.hpp"
#include "nearest_neighbour_dictionar.hpp"
#include <stack>
#include <map>
#include <set>
#include <utility>
#include <iostream>
#include <sstream> // for get_info method
#include <fstream>

namespace sdsl{

	//! A class that provides support for bit_vectors that represent a balanced parentheses sequence.
	/*! An opening parenthesis is represented by a 1 in the bit_vector and a closing parenthesis by a 0.
	 *  This class could be parametrized by a rank_support and select_support.
	 */	
	template<class RankSupport = rank_support_v<>, class SelectSupport = select_support_mcl<1>, 
		class NearestNeighbourDictionary = nearest_neighbour_dictionary<>, uint32_t block_size = 64>
	class balanced_parentheses_support_g{
		typedef bit_vector::size_type size_type;
		private:
			const bit_vector	*m_bp;		// the supported balanced parentheses sequence as bit_vector
			RankSupport			m_rank;		// a rank dictionary for calculation of excess
			SelectSupport 		*m_select;	// for the select-Operation

			//TODO bool is_toplevel erstellen oder irgendwie fuer sorgen, dass m_select nur da angelegt wird

			NearestNeighbourDictionary		m_pioneers_nnd;		// nnd for locating the pioneer-family

			size_type m_blocks;		// number of blocks
			size_type m_size;		// number of parentheses

			bit_vector						*m_pioneers;			// like m_bp, but only the pioneers
			balanced_parentheses_support_g	*m_pioneers_bpsg;		// rekursive data structure for the pioneers

/*			int_vector<> m_far_table	// Array which stores for every block for each parenthesis
										// where it's matching one is, if the matching one is in the
										// same block, otherwise block_size is stored for "far" parenthesis
*/

			//TODO: Enclose-Table

			void copy(const balanced_parentheses_support_j &bp_support) {} // TODO

			/*! get_matching_parenthesis
			 *  \param i index of the parenthesis to find its match, i.e. matching opening / closing parenthesis
			 *  \return index of the matching parenthesis, if in same block, otherwise return size()
			 */
			size_type get_matching_parenthesis(size_type i) {
				//TODO: implement with far-table
				
				size_type block_number = i / block_size;
				uint32_t excess = 0; // excess-value for equal-excess-search
				
				if ((*m_bp)[i] == 1) { // opening
					for(int j=i+1; j < (block_number+1) * block_size; j++) {
						if ((*m_bp)[j] == 1) excess++;
						else { // closing
							if (excess == 0) // matching parenthesis found
								return j;
							excess--;
						}
					}
					return m_size;
				} else { // closing
					for(int j=i-1; j >= block_number * block_size; j--) {
						if ((*m_bp)[j] == 1) {
							if (excess == 0) // matching parenthesis found
								return j;
							excess++;
						}
						else excess--;
					}
					return m_size;
				}
			}



			/*! getBlock
			 *  \param i position of parenthesis in question
			 *  \return the number of the block in which i is located
			 */
			size_type getBlock(size_type i) {
				return i / block_size;
			}
	
			/*! leftmost_closing
			 *  \param block number of the questionable block
			 *  \param excess_value excess-value of the wanted parenthesis
			 *  \return position of the leftmost closing parenthesis in the block with the given excess-value
			 */
			size_type leftmost_closing(size_type block, size_type excess_value) {} // TODO

			/*! rightmost_opening
			 *  \param block number of the questionable block
			 *  \param excess_value excess-value of the wanted parenthesis
			 *  \return position of the rightmost opening parenthesis in the block with the given excess-value
			 */
			size_type rightmost_opening(size_type block, size_type excess_value) {} // TODO

			/*! isEnclosedInSameBlock
			 *  \param i position of the parenthesis in question
			 *  \return 1 if and only if one of the most tightly enclosing parentheses of i is in the same block as i
			 */
			bool isEnclosedInSameBlock(size_type i) {} // TODO

			/*! getEnclosingParenthesis
			 *  \param i position of an Paranthesis which is enclosed in the same block (see isEnclosedInSameBlock(i))
			 *  \return one of the enclosing Parentheses if exsistent, else size()
			 */
			size_type getEnclosingParenthesis(size_type i) // TODO

			/*! first_far_parenthesis_left
			 *  \param i position of the parenthesis
			 *  \return the first far parenthesis left of i
			 */
			size_type first_far_parenthesis_left(size_type i) // TODO

			/*! rightmost_far_parenthesis_in_block
			 *  \param block
			 *  \return the reightmost far parenthesis is the given block
			 */
			size_type rightmost_far__parenthesis_in_block(size_type block) // TODO

		public:
			
			//! Constructor
			// TODO toplevel privatisiern
			balanced_parentheses_support_g(const bit_vector *bp = NULL, bool toplevel = 1): 
				m_bp(bp), m_size(bp==NULL?0:bp->size()) {

				m_blocks = (m_size+m_block_size-1)/m_block_size;

				if (toplevel) m_select = new SelectSupport(); // TODO find better condition

				// recursion-base: when to stop creating recursive pioneer data-structures:
				// TODO better base
				if (m_size <= block_size)
					return; // no recursion
					// TODO check whether all methods will work fine with that

				m_pioneers = new bit_vector();

				// pioneer bitmap:
				bit_vector pioneer_bitmap;		// bitmap of pioneers
				algorithm::calculate_pioneers_bitmap(m_bp, block_size, &pioneer_bitmap);

				// initialize nnd:
				m_pioneers_nnd = NearestNeighbourDictionary(pioneer_bitmap); // TODO c++ ?
				
				// create recursive data-structure for the pioneers:
				m_pioneers->resize(TODO); // TODO how many pioneers are there
				size_type pioneers_pos = 0;
				for (size_type i=0; i<m_size; i++)
					if (pioneer_bitmap[i] == 1)
						(*m_pioneers)[pioneers_pos++] = (*m_bp)[i];
				m_pioneers_bpsg = new balanced_parentheses_support_g(&m_pioneers); // create recursive data structure just now

				// TODO initialize far_table, if used

				// ...
			}


			//! Destructor
			// TODO
			~balanced_parentheses_support_g() {
				delete m_select;
				delete m_pioneers;
				delete m_pioneers_bpsg;
			}

			//! Copy constructor
			balanced_parentheses_support_g(const balanced_parentheses_support_g &bp_support){
				copy(bp_support);
			}

			//! Assignment operator
			balanced_parentheses_support_g& operator=(const balanced_parentheses_support_g &bp_support){
				if( this != &bp_support ){
					copy(bp_support);	
				}
				return *this;
			}
		
			/*! The size of the supported balanced parentheses sequence.
			 * \return the size of the supported balanced parentheses sequence. 
			 */
			size_type size() const{
				return m_size;
			}

			/*! Calculates the excess value at index i
			 * \param i The index of which the excess value should be calculated
			 * \return excess-value, i.e. the number of opening minus the number of closing parentheses
			 *   at position i, including index i
			 */
			size_type excess(size_type i) const {
				// ones = rank(i), zeros = i+1-ones
				// => ones - zeros = 2*ones - (i+1)
				return (rank(i)<<1)-i-1; 
			}

			/*! Returns the number of opening parentheses up to and including index i.
			 *  \pre{ \f$ 0 \leq i \leq size() \f$ } 
			 *  \return opening parentheses up to and inluding index i
			 */ 
			size_type rank(size_type i) const {
				// including position i
				return m_rank.rank(i+1);
			}

			/*! Returns the index of the i-th opening parenthesis.
			 *  \param i Number of the parenthesis to select. 
			 *  \pre{ \f$1\leq i < rank(size())\f$ }
			 *  \post{ \f$ 0\leq select(i) < size() \f$ }
			 *  \return index of i-th 1
			 */ 
			size_type select(size_type i) const {
				return m_select->select(i);
			}

			/*! Calculate the index of the matching closing parenthesis to the parenthesis at index i.
			 * \param i Index of an parenthesis. 0 <= i < size().
			 * \return * i, if the parenthesis at index i is closing,
			 *         * the position j of the matching closing parenthesis, if a matching parenthesis exists,
			 *         * size() if no matching closing parenthesis exists.
			 */ 
			// TODO
			size_type find_close(size_type i) const {
				// concept:
				//
				//       /--------------------------------------------------------\
				//       |													      |
				//       |														 \ /
				//       P														  P
				// ------(------------------(------------------)------------------)----
				//      l_p  <-exc_diff->   i			     y(i) <-exc-diff->  l_p_c
				//
				// is i a valid index?
				if (i >= m_size || i < 0)
					throw std::out_of_range("OUT_OF_RANGE: balanced_parentheses_support_g::find_close");
				// is i a closing parenthesis?
				if ((*m_bp)[i] == 0) return i;
				// if i is not a far parenthesis, return the value from the far-table or block-scan
					// TODO Hinweis: falls keine rekursive Unterdatenstruktur mehr da ist, ist hier nix far
				size_type far_entry = get_matching_parenthesis(i);
				if (far_entry != m_size)
					return far_entry;
				// find last pioneer befor i: 
				size_type last_pioneer = m_pioneers_nnd.prev(i);
				// find its position in the pioneer-family: 
				size_type last_pioneer_rank = m_pioneers_nnd.rank(last_pioneer);
				// find recursivly the matching closing parenthesis in the pioneer-family:
				size_type last_pioneer_close_rank = m_pioneers_bpsg->find_close(last_pioneer_rank);
				// calculate its position in the original bitvector:
				size_type last_pioneer_close = m_pioneers_nnd.select(last_pioneer_close_rank);
				// calculate difference betweet the excess-values of last_pioneer and i
				uint32_t excess_diff = excess(i) - excess(last_pioneer);
				// calculate absolut excess of i_close:
				uint32_t excess_i_close = excess(last_pioneer_close) - excess_diff;
				size_type close_block = last_pioneer_close / block_size;
				// find leftmost closing parenthesis in this block with the given excess-value:
				return leftmost_closing(close_block, excess_i_close);
			}

			//! Calculate the matching opening parenthesis to the closing parenthesis at position i
			/*! \param i Index of a closing parenthesis.
			  * \return * i, if the parenthesis at index i is closing,
			  *         * the position j of the matching opening parenthesis, if a matching parenthesis exists,
			  *         * size() if no matching closing parenthesis exists.
			  */
			// TODO
			size_type find_open(size_type i) const {
				// is i a valid index?
				if (i >= m_size || i < 0)
					throw std::out_of_range("OUT_OF_RANGE: balanced_parentheses_support_g::find_open");
				// is i a opening parenthesis?
				if ((*m_bp)[i] == 1) return i;
				// if i is not a far parenthesis, return the value from the far-table or block-scan
					// TODO Hinweis: falls keine rekursive Unterdatenstruktur mehr da ist, ist hier nix far
				size_type far_entry = get_matching_parenthesis(i);
				if (far_entry != m_size)
					return far_entry;
				// find the next pioneer: 
				size_type next_pioneer = m_pioneers_nnd.next(i);
				// find its position in the pioneer-family: 
				size_type next_pioneer_rank = m_pioneers_nnd.rank(next_pioneer);
				// find recursivly the matching opening parenthesis in the pioneer-family:
				size_type next_pioneer_open_rank = m_pioneers_bpsg.find_open(next_pioneer_rank);
				// calculate its position in the original bitvector:
				size_type next_pioneer_open = m_pioneers_nnd.select(next_pioneer_open_rank);
				// calculate difference betweet the excess-values of next_pioneer and i
				int32_t excess_diff = excess(next_pioneer) - excess(i);
				// calculate absolut excess of i_close:
				int32_t excess_i_open = excess(next_pioneer_open) + excess_diff;
				size_type open_block = next_pioneer_open / block_size;
				// find rightmost opening parenthesis in this block with the given excess-value:
				return rightmost_opening(open_block, excess_i_open);				
			}
			
			//! Calculate the index of the opening parenthesis corresponding to the closest matching parenthesis pair enclosing i.
			/*! \param i Index of an opening parenthesis.
			 *  \return The index of the opening parenthesis corresponding to the closest matching parenthesis pair enclosing i,
			 *          or size() if no such pair exists.
			 */	  
			// TODO
			size_type enclose(size_type i)const {
				// i should be an opening opening parenthesis 
				if ((*m_bp)[i] == 0) //closing
					i = find_open(i);
				// Tabelle befragen, ob einschliessende Klammer encl im aktuellen Block ist
				if (isEnclosedInSameBlock(i)){
					encl = getEnclosingParenthesis(i);
					if ((*m_bp)[encl] == 0) // closing
						encl = find_open(encl);
					return encl;
				}
				// pioneer dahinter suchen
				next_pioneer = m_pioneers_nnd.next(i);  //n_p
				encl_open_pioneer;						//e_o_p

				// Es folgt der Abschnitt um denjenigen pioneer zu finden, der i am engsten einschliesst
				//
				// Einfache Variante: der naechte pioneer ist schliessend, dann gehoert er zum
				// einschliessenden pioneer-paar
				//
				//         P      P   P                     P
				// --------(------(---)---------(-----------)--
				//       e_o_p                  i          n_p
				//
				if ((*m_bp)[next_pioneer] == 0) // closing
					encl_open_pioneer = findopen(next_pioneer);
				//
				//
				// kompliziertere Variante: der naechte pioneer ist oeffnend, so muss ueber das pioneer_nnd
				// der vorgaenger-pioneer bestimmt werden. dieser ist zwingend oeffnend )
				//
				//         P      P   P                     P
				// --------(------(---)---------(-----------(--
				//       e_o_p                  i          n_p
				else {
					// Position in der Pioneer-Family bestimmen
					next_pioneer_rank = m_pioneers_nnd.rank(next_pioneer);
					// rekursiv enclose finden
					encl_open_pioneer_rank = m_pioneers_bpsg.enclose(next_pioneer_rank);
					// rueckuebersetzung ins aktuelle bpsg
					encl_open_pioneer = select(encl_open_pioneer_rank);
				}
				
				
				// TODO: Nachfolgendes verstehen
				
				q = m_pioneers_nnd.next(encl_open_pioneer+1); // +1, damit es auf jeden Fall den nächsten trifft

				if (getBlock(q) == getBlock(encl_open_pioneer))
					return first_far_parenthesis_left(q);
				else
					return rightmost_far_paranthesis_in_block(getBlock(encl_open_pioneer));
			}

			//! Serializes the balanced_parentheses_support_j to a stream.
			/*!
			 * \param out The outstream to which the data structure is written.   
			 * \return The number of bytes written to out.
			 */
			// TODO
			size_type serialize(std::ostream &out) const{
				/* size_type written_bytes = 0;
				written_bytes += m_rank.serialize(out);
				written_bytes += m_select.serialize(out);

				written_bytes += m_pioneer_bitmap.serialize(out);
				written_bytes += m_rank_pioneer_bitmap.serialize(out);				
				written_bytes += m_pioneer.serialize(out);

				written_bytes += m_enclose_pioneer_bitmap.serialize(out);
				written_bytes += m_rank_enclose_pioneer_bitmap.serialize(out);
				written_bytes += m_enclose_pioneer.serialize(out);

				return written_bytes;
				*/
			}

			//! Load the balanced_parentheses_support_j for a bit_vector v.
			/*!
			 * \param in The instream from which the data strucutre is read.
			 * \param bp Bit vector representing a balanced parentheses sequence that is supported by this data structure. 
			 */
			// TODO
			void load(std::istream &in, const bit_vector *bp){
				/* m_bp = bp;
				m_size = m_bp->size();
				m_block_size = 64;
				m_blocks = (m_size+m_block_size-1)/m_block_size;

				m_rank.load(in, m_bp);
				m_select.load(in, m_bp);

				m_pioneer_bitmap.load(in);
				m_rank_pioneer_bitmap.load(in, &m_pioneer_bitmap);
				m_pioneer.load(in);

				m_enclose_pioneer_bitmap.load(in);
				m_rank_enclose_pioneer_bitmap.load(in, &m_enclose_pioneer_bitmap);
				m_enclose_pioneer.load(in);
				*/
			}	

			// TODO
			std::string get_info()const{
				/* std::stringstream ss;
				ss<<"number of parentheses: "<<m_bp->size()<<std::endl;
				ss<<"number of pioneers: "<< m_pioneer.size()<<std::endl; 
				std::ofstream out("/dev/null");
				ss<<"size of parentheses sequence: "<< m_bp->serialize(out) << std::endl;
				ss<<"size of pioneer bitmap: "<< m_pioneer_bitmap.serialize(out) << std::endl;
				ss<<"size of pioneer data  : "<< m_pioneer.serialize(out) << std::endl;
				ss<<"size of rank for pioneers: "<< m_rank_pioneer_bitmap.serialize(out) << std::endl;
				
				ss<<"size of enclose  pioneer bitmap: "<< m_enclose_pioneer_bitmap.serialize(out) << std::endl;
				ss<<"size of pioneer data  : "<< m_pioneer.serialize(out) << std::endl;
				ss<<"size of rank for pioneers: "<< m_rank_enclose_pioneer_bitmap.serialize(out) << std::endl;

				return ss.str();
				*/
			}
	};
}

#endif
