/* sdsl - succinct data structures library
    Copyright (C) 2009 Simon Gog 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see http://www.gnu.org/licenses/ .
*/
/*! \file lcp.hpp
    \brief lcp.hpp contains classes for lcp information. 
	\author Simon Gog
*/
#ifndef INCLUDED_SDSL_LCP
#define INCLUDED_SDSL_LCP

#include "int_vector.hpp"

//! Namespace for the succinct data structure library.
namespace sdsl{

template<class Csa, class Lcp>
struct lcp_trait{
	static void construct_lcp(Lcp &lcp, const Csa &csa, const unsigned char *text){
		lcp = Lcp(&csa, text);	
	}
	static void copy_lcp(Lcp &lcp, const Lcp &oldlcp, const Csa &csa){
		lcp = oldlcp;
		lcp.set_csa(&csa);	
	}
	static void load_lcp(Lcp &lcp, std::istream &in, const Csa &csa){
		lcp.load(in, &csa);	
	}
};

template<class Csa, uint8_t fixedIntWidth>
struct lcp_trait<Csa, int_vector<fixedIntWidth> >{
	static void construct_lcp(int_vector<fixedIntWidth> &lcp, const Csa &csa, const unsigned char *text){
		lcp.setIntWidth( bit_magic::l1BP(csa.size())+1 );
		lcp.resize(csa.size());
		typename Csa::size_type sa_1 = csa(0), l=0, oldl = 0;	
		for(typename Csa::size_type i=0,j=0, size=csa.size(); i < size; ++i){
			oldl = l;
			if(l) --l;
			if( sa_1 ){
				j = csa[sa_1-1];	
				while(i+l < size and j+l < size and text[i+l]==text[j+l]) ++l;
			}
			else{
				l = 0;
			}
			lcp[sa_1] = l;
			sa_1 = csa.psi[sa_1];
		}
	}

	static void copy_lcp(int_vector<fixedIntWidth> &lcp, const int_vector<fixedIntWidth> &oldlcp, const Csa &csa){
		lcp = oldlcp; 
	}

	static void load_lcp(int_vector<fixedIntWidth> &lcp, std::istream &in, const Csa &csa){
		lcp.load(in); 
	}
};



} // end namespace sdsl

#include "lcp_sada.hpp"
#include "lcp_kurtz.hpp"

#endif
