/* sdsl - succinct data structures library
    Copyright (C) 2009 Simon Gog 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see http://www.gnu.org/licenses/ .
*/
/*! \file suffixtrees.hpp
    \brief suffixtrees.hpp contains generic classes for different suffix tree classes.
	\author Simon Gog
*/
#ifndef INCLUDED_SDSL_SUFFIXTREES
#define INCLUDED_SDSL_SUFFIXTREES

/** \defgroup cst Compressed Suffix Trees (CST) 
 *   This group contains data structures for compressed suffix trees. The following methods are supported:
 *    - root
 *    - child(v,c)
 *    - ith_child(v)
 *    - ith_leaf(i)
 *    - parent(v)
 *    - sl(v)
 *    - lca(v,w)
 *    - ..
 */

#include "cst_sct.hpp"
#include "cst_sada.hpp"
#include "cst_sct2.hpp"

#include "csa_uncompressed.hpp"
#include "int_vector.hpp"

namespace sdsl{

typedef cst_sct<csa_uncompressed, int_vector<0> > cst_sct_uncompressed;
typedef cst_sada<csa_uncompressed, int_vector<0> > cst_sada_uncompressed;

}


#endif
