#include <sdsl/suffixarrays.hpp>
#include <sdsl/suffixtrees.hpp>
#include <iostream>
#include <string>
#include <iomanip>

using namespace sdsl;
using namespace std;

unsigned char replace_sentinel(unsigned char c){
	if(c==0)
		return '$';
	return c;
}

unsigned char vc(unsigned char c){
	if(c=='!')
		return '_';
	else
		return '!';
}

template<class csa_type>
void test_csa(const unsigned char * text){
	typedef typename csa_type::size_type size_type;
	string s((const char*)text);
	cout << "        ";
	for(size_type i=0; i < s.size(); ++i){
		cout<< (i%10);
	}
	cout<<endl;
	cout << "text = \"" << text << "\"" << endl;
	csa_type csa(text);
	cout << "csa info " << endl;
	cout << "text size: " << (int)csa.size() << endl;
	cout << "alphabet size: " << (int)csa.sigma << endl;
	cout << "C: ";
	for(size_type i=0; i<csa.sigma; ++i){
		cout << csa.C[i] << " ";
	}
	cout << endl;
	cout <<"   i  SA iSA psi  LF BWT  #" << text[0] << endl;
	for(size_type i=0; i<csa.size(); ++i){
		cout << setw(4) << i << setw(4) << csa[i] << setw(4) << csa(i) << setw(4) << csa.psi[i] << setw(4) << csa.psi(i) 
			 << setw(4) << replace_sentinel(csa.bwt[i]) << setw(4) << csa.rank_bwt(i+1,text[0]) << " " << text+csa[i] << endl;
	}
	for(size_type i=1; i <= csa.C[2]-csa.C[1]; ++i ){
		cout << i <<"th occurence of "<<csa.comp2char[1] << " : "<< csa.select_bwt(i, csa.comp2char[1]) << endl;
	}
	size_type t;
	typedef cst_sct<csa_type> cst_type;
//	cst_sct<csa_type> cst((const unsigned char*)text);
	cst_type cst((const unsigned char*)text);
//	typedef typename cst_sct<csa_type>::node_type node_type;
	typedef typename cst_type::node_type node_type;
	cout << cst.root() << endl;
	cout << cst.child(cst.root(),'a', t) << endl;
	cout << cst.child(cst.child(cst.root(),'a', t), 'b', t) << endl;
	node_type n1 = cst.child(cst.child(cst.root(),'a', t), 'b', t);
	cout << "wl("<<n1<<","<<'a'<<")="<< cst.wl(n1, 'a') << endl;
	node_type n2 = cst.child(cst.root(), 'b', t);
	cout << "wl("<<n2<<","<<'a'<<")="<< cst.wl(n2, 'a') << endl;
	cout << "wl("<<cst.root()<<","<<'a'<<")="<< cst.wl(cst.root(), 'a') << endl;
	node_type n3 = cst.child(n2, '\0', t);
	cout << "n3 = " << n3 << endl;
	cout << "wl(n3,'a') = " << cst.wl(n3,'a') << endl; 
	cout << "wl(n3,'b') = " << cst.wl(n3,'b') << endl;
	for(typename cst_type::iterator it = cst.begin(); it!=cst.end(); ++it){
		for(size_type i=0; i<cst.depth(*it); ++i)
			cout<<" ";
		cout<<*it<<endl;
	}

	// output suffix tree in tikz-format
	for(typename cst_type::iterator it = cst.begin(); it!=cst.end(); ++it){
		if( it.visit()==1 ){
			double f = 1.5;
			double fy = 1.3;
			string style = cst.is_leaf(*it) ? "leafnode" : "innernode";
			double xpos = (cst.rightmost_suffix_array_index_in_the_subtree(*it) + 
						   cst.leftmost_suffix_array_index_in_the_subtree(*it));
			double ypos = cst.depth(*it);
			cout<<"\\node["<<style<<"="<<f*cst.leaves_in_the_subtree(*it)-0.2<<"cm] (node "<<ypos<<"x"<<xpos<<") at ("<<f*xpos/2<<","<<-fy*ypos<<") {"<<*it<<"};"<<endl;
			
			if( *it != cst.root() ){
				node_type p =  cst.parent(*it);
				double pypos = cst.depth(p);
				cout<<"\\draw[->] ("<<f*xpos/2<<","<<-fy*pypos<<") -- (node "<<ypos<<"x"<<xpos<<".north);"<<endl;

				for(size_type i=cst.depth(p)+1; i <= cst.depth(*it); ++i){
//					cout << replace_sentinel(cst.edge(*it, i)) << endl;
					unsigned char c = replace_sentinel(cst.edge(*it,i));
					double y = -fy*(pypos -1 + i-cst.depth(p) + 0.5 );
					cout << "\\node[anchor=south east] at ("<<f*xpos/2<<","<<y<<"){\\verb"<<vc(c)<<c<<vc(c)<<"};"<<endl;
				}
			}
		}
	}	

}

int main(){
	typedef csa_wt<> csa_type;
	const unsigned char * text = (const unsigned char*)"abaaaabaaababaaabaaa";	
	test_csa<csa_wt<> >(text);
	test_csa<csa_sada_prac<> >(text);
}
