#include <sdsl/suffixarrays.hpp>
#include <sdsl/util.hpp>
#include <sdsl/testutils.hpp>
#include <string>
#include <cstdio>
#include <vector>

using namespace sdsl;
using namespace std;

typedef vector<int> tVI;

// DEBUG OUT
#define DOUT(x) cout << #x << "\t=\t" << x << endl

typedef bit_vector::size_type size_type;

template<class Csa>
void time_psi(const Csa &csa, size_type &t){
	t=0;
	stop_watch sw;
	sw.start();
	for(size_type i=0,x=123; i<csa.size(); ++i){
		x = (x*x-1)%csa.size();
		t += csa.psi[x];
	}
	sw.stop();
	cout << sw.getUserTime() << " t = " << t <<endl; 
}

template<class Csa>
void time_sa(const Csa &csa, size_type &t){
	t=0;
	stop_watch sw;
	sw.start();
	for(size_type i=0,x=123; i<csa.size(); ++i){
		x = (x*x-1)%csa.size();
		t += csa[x];
	}
	sw.stop();
	cout << sw.getUserTime() << " t = " << t <<endl; 
}

template<class Csa>
void time_isa(const Csa &csa, size_type &t){
	t=0;
	stop_watch sw;
	sw.start();
	for(size_type i=0,x=123; i<csa.size(); ++i){
		x = (x*x-1)%csa.size();
		t += csa(x);
	}
	sw.stop();
	cout << sw.getUserTime() << " t = " << t <<endl; 
}

template<class Csa>
void time_bwt_occ(const Csa &csa, size_type &t){
	t=0;
	stop_watch sw;
	sw.start();
	for(size_type i=0; i<csa.size(); ++i){
		t += csa.occ_bwt(i,"a");
	}
	sw.stop();
	cout << sw.getUserTime() << " t = " << t <<endl; 
}

template<class Csa>
void get_psi_run_len(const Csa &csa){
	int *count = new int[csa.get_psi_sample_dens()+1];
	int sum = 0;
	for(uint32_t i=0; i<csa.get_psi_sample_dens()+1; ++i)
		count[i] = 0;

	for(size_type i=1,c=0, inc=1, last_val=csa.psi[0]; i <csa.size(); ++i){
		if(	i % csa.get_psi_sample_dens() == 0 ){
			count[c]++;
			sum++;
			c = 0;
			inc=1;
		}
		size_type psi = csa.psi[i];
		if( inc and psi == last_val+1 )
			++c;
		else
			inc=0;
		last_val = psi;
	}
	cout << "psi_sample_dens = " << csa.get_psi_sample_dens() << endl;
	for(size_type i=0; i<=csa.get_psi_sample_dens();++i){
		cout << setw(4) << i << " " << count[i] << "    " << ((double)count[i])/sum << endl;
	}
	delete [] count;

	size_type len=0;
	for(size_type i=0; i<csa.size();++i){
		size_type j=i;
		len++;
		while( j%csa.get_psi_sample_dens() != 0 )	{
			++len;
			j = csa.psi[j];
		}
	}
	cout << "avg cycle len = " << ((double)len)/csa.size() << endl;
}

int main(){

	char *c=NULL;
	file::readText("@TESTCASESDIR@/small/dblp.xml.1MB", c);	
//	file::readText("@TESTCASESDIR@/small/dna.1MB", c);	
//	file::readText("@TESTCASESDIR@/small/english.1MB", c);	
	if( c!=NULL ){
		stop_watch sw;
		size_type t;

		csa_sada_prac<enc_vector_prac<enc_vector_prac<>::coder, 8> > 	csa1((const unsigned char*)c);
		csa_wt<> 			csa2((const unsigned char*)c);
		csa_uncompressed	csa3((const unsigned char*)c);

		get_psi_run_len(csa1);

		DOUT( csa1.size() ); 
		DOUT( csa2.size() ); 
		DOUT( csa3.size() ); 
		DOUT( (int)csa1.sigma );
		DOUT( (int)csa2.sigma );
		DOUT( (int)csa3.sigma );
		DOUT( (int)csa1.get_psi_sample_dens() );
		
		DOUT( util::get_size_in_bytes(csa1) );
		DOUT( util::get_size_in_bytes(csa2) );
		DOUT( util::get_size_in_bytes(csa3) );

		for(size_type i=0; i<csa1.size(); ++i){
			size_type p1,p2,p3;
			p1 = csa1.psi[i];
			p2 = csa2.psi[i];
			p3 = csa3.psi[i];
			if( p1 != p2 or p2 != p3 ){
				cout << "ERROR" << endl;
				DOUT(i ); DOUT( p1 ); DOUT( p2 ); DOUT( p3 );
			}
		}
	
		cout << "csa_sada_prac" << endl;

		time_sa( csa1, t );
		time_isa( csa1, t );
		time_psi( csa1, t );

goto finish;
		cout << "csa_wt" << endl;

		time_sa( csa2, t );
		time_psi( csa2, t );
		time_isa( csa2, t );

		cout << "csa_uncompressed" << endl;

		time_sa( csa3, t );
		time_isa( csa3, t );
		time_psi( csa3, t );



		for(size_type i=0; i<csa1.size(); ++i){
			for(size_type ci=0; ci<csa1.sigma; ++ci){
					size_type r1,r2,r3;
					r1 = csa1.rank_bwt(i+1, csa1.comp2char[ci]);
					r2 = csa2.rank_bwt(i+1, csa2.comp2char[ci]);
					r3 = csa3.rank_bwt(i+1, csa3.comp2char[ci]);
					if( r1 != r2 or r2 != r3 ){
						cout << "ERROR" << endl;
						DOUT( i ); DOUT( ci ); DOUT( r1 ); DOUT( r2 );  DOUT( r3 );
						DOUT( csa1.comp2char[ci] );
						DOUT( csa2.comp2char[ci] );
						DOUT( csa3.comp2char[ci] );
						DOUT( csa2.C[ci] );
						DOUT( csa2.C[ci+1] );
						goto finish;				
					}
			}
		}
		t=0;
		sw.start();
		for(size_type i=0; i<csa1.size(); ++i){
			for(size_type ci=0; ci<csa1.sigma; ++ci){
				t += csa1.rank_bwt(i+1, csa1.comp2char[ci]);
			}
		}
		sw.stop();
		cout<<" cst1 time for rank_bwt =  "<<sw.getUserTime()<<endl;
		sw.start();
		for(size_type i=0; i<csa2.size(); ++i){
			for(size_type ci=0; ci<csa2.sigma; ++ci){
				t -= csa2.rank_bwt(i+1, csa2.comp2char[ci]);
			}
		}
		sw.stop();
		cout<<" cst2 time for rank_bwt =  "<<sw.getUserTime()<<endl;
		if( t!=0 ){
			cout << "ERROR" << endl;
		}
		sw.start();
		for(size_type i=0; i<csa3.size(); ++i){
			for(size_type ci=0; ci<csa3.sigma; ++ci){
				t -= csa3.rank_bwt(i+1, csa3.comp2char[ci]);
			}
		}
		sw.stop();
		cout<<" cst3 time for rank_bwt =  "<<sw.getUserTime()<<" t="<<t<<endl;

		t = 0;
		size_type ccount[256];
		for(size_type i=0; i<256;++i) ccount[i]=0;
		sw.start();
		for(size_type i=0; i<csa1.size(); ++i){
			++ccount[(int)c[i]];
			t += csa1.select_bwt(ccount[(int)c[i]], c[i]);
		}
		sw.stop();
		cout<<" cst1 time for select_bwt =  "<<sw.getUserTime()<<endl;

		for(size_type i=0; i<256;++i) ccount[i]=0;
		sw.start();
		for(size_type i=0; i<csa2.size(); ++i){
			++ccount[(int)c[i]];
			t -= csa2.select_bwt(ccount[(int)c[i]], c[i]);
		}
		sw.stop();
		cout<<" cst2 time for select_bwt =  "<<sw.getUserTime()<<endl;

		if( t!=0 ){
			cout << "ERROR" << endl;
		}

		for(size_type i=0; i<256;++i) ccount[i]=0;
		sw.start();
		for(size_type i=0; i<csa3.size(); ++i){
			++ccount[(int)c[i]];
			t -= csa3.select_bwt(ccount[(int)c[i]], c[i]);
		}
		sw.stop();
		cout<<" cst3 time for select_bwt =  "<<sw.getUserTime()<<" t="<<t<<endl;

		t = 0;

		sw.start();
		for(size_type i=0; i<csa1.size(); ++i){
			t += csa1.bwt[i];
		}
		sw.stop();
		cout<<" cst1 time for bwt =  "<<sw.getUserTime()<<endl;

		for(size_type i=0; i<csa1.size(); ++i){
			const unsigned char b1 = csa1.bwt[i];
			const unsigned char b2 = csa2.bwt[i];
			if( b1 != b2 ){
				DOUT( i );
				DOUT( b1 );
				DOUT( b2 );
				goto finish;
			}
		}

		sw.start();
		for(size_type i=0; i<csa2.size(); ++i){
			t -= csa2.bwt[i];
		}
		sw.stop();
		cout<<" cst2 time for bwt =  "<<sw.getUserTime()<<endl;

		if( t!=0 ){
			cout << "ERROR" << endl;
		}


		sw.start();
		for(size_type i=0; i<csa3.size(); ++i){
			t -= csa3.bwt[i];
		}
		sw.stop();
		cout<<" cst2 time for bwt =  "<<sw.getUserTime()<< "t=" << t <<endl;



finish:		
		delete [] c;
	}
}
