#define BOOST_TEST_DYN_LINK
#define BOOST_TEST_MAIN 
#include <boost/test/unit_test.hpp>
#include <sdsl/algorithms.hpp> 
#include <cstring>
#include <vector>

#if BOOST_WORKAROUND( __GNUC__, < 3 )
typedef boost::test_tools::output_test_stream onullstream_type;
#else
#include <boost/test/utils/nullstream.hpp>
typedef boost::onullstream onullstream_type;
#endif

using namespace boost::unit_test;
using namespace sdsl;

typedef std::vector<uint32_t> tVI;

BOOST_AUTO_TEST_SUITE( algorithm_test_suite )

BOOST_AUTO_TEST_CASE( suffix_array_construction_test ){
	unit_test_log.set_threshold_level( log_messages );
	BOOST_TEST_MESSAGE( "Running tests for the suffix array construction of algorithm..." );
	const unsigned char *text1 = (unsigned char*)"swissmissismissing";
	const unsigned char *text2 = (unsigned char*)"umulmundumulmundumulmundumulmherum";
	unsigned char	   *text3 = new unsigned char[10001];
	unsigned char 	   *text4 = new unsigned char[1000001];
	size_t len1 = strlen((char*)text1),
		   len2 = strlen((char*)text2),
		   len3 = 10000,
		   len4 = 1000001;

	for(size_t i=0;i<len3; ++i) text3[i] = 'a';
	text3[len3] = '\0';
	for(size_t i=0;i<len4; ++i) text4[i] = 'a'+rand()%26;
	tVI sa1(len1+1), sa2(len2+1), sa3(len3+1), sa4(len4+1);
	int_vector<> sab1(len1+1, 0, bit_magic::l1BP(len1+1)+1),
				 sab2(len2+1, 0, bit_magic::l1BP(len2+1)+1),
				 sab3(len3+1, 0, bit_magic::l1BP(len3+1)+1);
	algorithm::calculate_sa(text1, len1+1, sa1);
	algorithm::calculate_sa(text1, len1+1, sab1);
//	for(size_t i=0; i<len1; ++i) BOOST_MESSAGE(sa1[i]<<" "<<text1+sa1[i]);
	BOOST_CHECK( algorithm::verify_sa(text1, len1+1, sa1) );
	BOOST_CHECK( algorithm::verify_sa(text1, len1+1, sab1) );

	algorithm::calculate_sa(text2, len2+1, sa2);
	algorithm::calculate_sa(text2, len2+1, sab2);
//	for(size_t i=0; i<len2; ++i) BOOST_MESSAGE(sa2[i]<<" "<<text2+sa2[i]);
	BOOST_CHECK( algorithm::verify_sa(text2, len2+1, sa2) );
	BOOST_CHECK( algorithm::verify_sa(text2, len2+1, sab2) );

	algorithm::calculate_sa(text3, len3+1, sa3);
	algorithm::calculate_sa(text3, len3+1, sab3);
	BOOST_CHECK( algorithm::verify_sa(text3, len3+1, sa3) );
	BOOST_CHECK( algorithm::verify_sa(text3, len3+1, sab3) );

	algorithm::calculate_sa(text4, len4+1, sa4);
	BOOST_CHECK( algorithm::verify_sa(text4, len4+1, sa4) );
}

BOOST_AUTO_TEST_SUITE_END()
