#define BOOST_TEST_DYN_LINK
#define BOOST_TEST_MAIN 
#include <boost/test/unit_test.hpp>
#include <sdsl/bitmagic.hpp>
#include <sdsl/vectors.hpp>

using namespace boost::unit_test;
using namespace boost::test_tools;
using namespace sdsl;

BOOST_AUTO_TEST_SUITE( bit_magic_test_suite )

BOOST_AUTO_TEST_CASE( correctness_test ){
	unit_test_log.set_threshold_level( log_messages );
	BOOST_MESSAGE( "Running correctness test for bit_magic..." );
	int_vector<64> v(1000000);
	util::set_random_bits(v);
	BOOST_MESSAGE( "Test b11Cnt" );
	uint64_t carry = 0;
	for(size_t i=0; i<v.size(); ++i){
		uint64_t t = v[i];
		uint32_t cnt=carry;
		uint32_t cnt11 = 0;
		for(size_t j=0;j<64;++j){
			if((t&1)==1)
				++cnt;
			else
				cnt=0;
			if(cnt==2){
				cnt11++;
				cnt=0;
			}
			t>>=1;
		}
		BOOST_REQUIRE( bit_magic::b11Cnt(v[i],carry) == cnt11 ); 
		carry = cnt;
	}

	BOOST_MESSAGE("Test b10Cnt");
	uint64_t carry2 = 0;
	carry = 0;
	for(size_t i=0; i<v.size(); ++i){
		uint64_t t = v[i];
		BOOST_REQUIRE( bit_magic::b10Cnt(t, carry) == bit_magic::b10CntNaive(t, carry2) );
		BOOST_REQUIRE( carry2 == carry );
	}

	BOOST_MESSAGE( "Test r1BP" );
	for(uint64_t x=1; x<1000000; x+=100){
		uint64_t y=x;
		uint32_t res=0;
		while(y%2==0 and res<64){
			res++;
			y/=2;
		}	
		res%=64;
		BOOST_REQUIRE(bit_magic::r1BP(x)==res);
	}

	BOOST_MESSAGE( "Test i1BP" );

	for(uint64_t i=1; i<=64; ++i){// easy cases
		BOOST_REQUIRE(bit_magic::i1BP(0xFFFFFFFFFFFFFFFFULL, i)==i-1);
		BOOST_REQUIRE(bit_magic::j1BP(0xFFFFFFFFFFFFFFFFULL, i)==i-1);
	}
	for(uint64_t i=0, x=0x0123456789ABCDEFULL, ones=0;i<64;++i)// One hand crafted test case
		if(x&(1ULL<<i)){ 
			++ones;
			BOOST_REQUIRE(bit_magic::i1BP(x,ones)==i);
//			if(bit_magic::j1BP(x,ones)!=i){
//			BOOST_MESSAGE("x="<<x<<" j="<<ones<<" expected = "<<i<<"  returned "<<bit_magic::j1BP(x,ones));
//			}
			BOOST_REQUIRE(bit_magic::j1BP(x,ones)==i);
		}
	for(uint64_t k=0; k<10000; ++k )// 10000 random test cases
		for(uint64_t i=0, x = (((uint64_t)rand())<<32) + rand(), ones=0;i<64;++i)
			if(x&(1ULL<<i)){
				++ones;
				BOOST_REQUIRE(bit_magic::i1BP(x,ones)==i);
				BOOST_REQUIRE(bit_magic::j1BP(x,ones)==i);
			}



	BOOST_MESSAGE( "Test l11BP" );
	// Four hand crafted test cases
	BOOST_REQUIRE(bit_magic::l11BP(1)==0);
	BOOST_REQUIRE(bit_magic::l11BP(0x5555555555555555ULL)==0);
	BOOST_REQUIRE(bit_magic::l11BP(0x65F55555555578FFULL)==62);
	BOOST_REQUIRE(bit_magic::l11BP(0x75F55555555578FFULL)==61);
	for(uint64_t i=0, x=3; i<63; ++i){ // easy cases
		BOOST_REQUIRE(bit_magic::l11BP(x)==i+1);
		x<<=1;
	}
	for(uint64_t k=0; k<10000; ++k){ // 10000 random test cases
		uint64_t x = ((uint64_t)rand()<<32)+rand();
		for(int16_t i=62;i>=0;--i){
			if( (x&&(3ULL<<i)) == (3ULL<<i) ){
				BOOST_REQUIRE(bit_magic::l11BP(x)==(uint32_t)i+1);
				break;
			}
		}
	}
		
	BOOST_MESSAGE( "Test i11BP" );
	// Three hand crafted test cases
	BOOST_REQUIRE(bit_magic::i11BP(3, 1, 0)==1); // 3 = 11
	BOOST_REQUIRE(bit_magic::i11BP(465489262,6,0)==20); // 465489262 == 11011101111101100110101101110
	BOOST_REQUIRE(bit_magic::i11BP(465489262,7,0)==24); // 465489262 == 11011101111101100110101101110

	for(uint64_t k=0; k<10000; ++k )// 10000 random test cases
		for(uint64_t i=0, x = (((uint64_t)rand())<<32) + rand(), ones=0;i<64;++i)
			if(((x&(3ULL<<i))>>i)==3){
				++ones;
				BOOST_REQUIRE(bit_magic::i11BP(x,ones)==i+1);
				++i;
			}

	BOOST_MESSAGE( "Test max_excess" );
	uint16_t temp;
	BOOST_REQUIRE(bit_magic::max_excess(0xFFFFFFFFFFFFFFFFULL, temp)==64 and temp == 64);
	BOOST_REQUIRE(bit_magic::max_excess(0x00000000FFFFFFFFULL, temp)==32 and temp == 32);
	BOOST_REQUIRE(bit_magic::max_excess(0x0000000000000000ULL, temp)==0 and temp == 0);

	for(uint64_t k=0; k<1000000; ++k ){//  random test cases
		uint64_t x = (((uint64_t)rand())<<32) + rand();
		uint16_t max_excess_val = 64, excess=64;
		for(int j=0;j<64;++j)
			if((x>>j)&1){
				++excess;
				if(excess>max_excess_val) max_excess_val = excess;
			}else
				--excess;
			max_excess_val -= 64;
			BOOST_REQUIRE(bit_magic::max_excess(x, temp)==max_excess_val);
			BOOST_REQUIRE(bit_magic::b1Cnt(x)==temp);
		}

	BOOST_MESSAGE( "Test max_excess2" );
	BOOST_REQUIRE(bit_magic::max_excess2(0xFFFFFFFFFFFFFFFFULL, temp)==64 and temp == 64);
	BOOST_REQUIRE(bit_magic::max_excess2(0x00000000FFFFFFFFULL, temp)==32 and temp == 32);
	BOOST_REQUIRE(bit_magic::max_excess2(0x0000000000000000ULL, temp)==0 and temp == 0);

	for(uint64_t k=0; k<1000000; ++k ){//  random test cases
		uint64_t x = (((uint64_t)rand())<<32) + rand();
		uint16_t max_excess_val = 64, excess=64;
		for(int j=0;j<64;++j)
			if((x>>j)&1){
				++excess;
				if(excess>max_excess_val) max_excess_val = excess;
			}else
				--excess;
			max_excess_val -= 64;
			BOOST_REQUIRE(bit_magic::max_excess2(x, temp)==max_excess_val);
			BOOST_REQUIRE(bit_magic::b1Cnt(x)==temp);
		}

	BOOST_MESSAGE( "Test max_excess3" );
	BOOST_REQUIRE(bit_magic::max_excess3(0xFFFFFFFFFFFFFFFFULL, temp)==64 and temp == 64);
	BOOST_REQUIRE(bit_magic::max_excess3(0x00000000FFFFFFFFULL, temp)==32 and temp == 32);
	BOOST_REQUIRE(bit_magic::max_excess3(0x0000000000000000ULL, temp)==0 and temp == 0);

	for(uint64_t k=0; k<1000000; ++k ){//  random test cases
		uint64_t x = (((uint64_t)rand())<<32) + rand();
		uint16_t max_excess_val = 64, excess=64;
		for(int j=0;j<64;++j)
			if((x>>j)&1){
				++excess;
				if(excess>max_excess_val) max_excess_val = excess;
			}else
				--excess;
			max_excess_val -= 64;
			BOOST_REQUIRE(bit_magic::max_excess3(x, temp)==max_excess_val);
			BOOST_REQUIRE(bit_magic::b1Cnt(x)==temp);
		}

/*	for(uint64_t k=0;k<256;++k){
		std::cerr<<bit_magic::max_excess(k, temp)<<",";
		if(k and k%8==7) std::cerr<<std::endl;
	}
*/

	uint64_t temp2, temp3;
	BOOST_MESSAGE( "Test first_excess_position" );
	BOOST_REQUIRE(bit_magic::first_excess_position(0xFFFFFFFFFFFFFFFFULL, 64, temp2)==63 and (temp2>>57) == 64);
	BOOST_REQUIRE(bit_magic::first_excess_position(0x00000000FFFFFFFFULL, 32, temp2)==31 and (temp2>>57) == 32);
	BOOST_REQUIRE(bit_magic::first_excess_position(0x0000000000000000ULL, 1, temp2)==64 and temp2 == 0);

	for(uint64_t k=0; k<100000; ++k ){//  random test cases
		uint64_t x = (((uint64_t)rand())<<32) + rand();
		for(int j=1;j<64;++j){
			BOOST_REQUIRE(bit_magic::first_excess_position(x, j, temp2)==bit_magic::first_excess_position_naive(x, j, temp3));
			BOOST_REQUIRE(temp2==temp3);
		}
	}
	
	
	BOOST_MESSAGE( "Test last_excess_position" );
	BOOST_REQUIRE(bit_magic::last_excess_position(0xFFFFFFFFFFFFFFFFULL, 64, temp2)==63 and (temp2>>57) == 64);
	BOOST_REQUIRE(bit_magic::last_excess_position(0x00000000FFFFFFFFULL, 32, temp2)==31 and (temp2>>57) == 32);
	BOOST_REQUIRE(bit_magic::last_excess_position(0x0000000000000000ULL, 1, temp2)==64 and temp2 == 0);
	BOOST_REQUIRE(bit_magic::last_excess_position(0x000000000000FF01ULL, 1, temp2)==16 and (temp2>>57) == 9);

	for(uint64_t k=0; k<100000; ++k ){//  random test cases
		uint64_t x = (((uint64_t)rand())<<32) + rand();
		for(int j=1;j<64;++j){
			uint8_t res = bit_magic::last_excess_position_naive(x,j,temp3);
			uint8_t res2 = bit_magic::last_excess_position(x, j, temp2);
			if( res!=res2){
				BOOST_MESSAGE("x="<<x<<" j="<<j<<" res="<<(int)res<<" res2="<<(int)res2);
				BOOST_REQUIRE(res==res2);
			}
			BOOST_REQUIRE(temp2==temp3);
		}
	}

//	bit_magic::generate_very_near_find_open();	
//	bit_magic::generate_last_pos_of_excess_val();	
	BOOST_MESSAGE( "Test find_open" );
	BOOST_REQUIRE(bit_magic::find_open(0x1ULL, 1)==0);
	BOOST_REQUIRE(bit_magic::find_open(0x3ULL, 3)==0);
	BOOST_REQUIRE(bit_magic::find_open(0x3ULL, 2)==1);
	BOOST_REQUIRE(bit_magic::find_open(0xFULL, 4)==3);
	BOOST_REQUIRE(bit_magic::find_open(0xFULL, 5)==2);
	BOOST_REQUIRE(bit_magic::find_open(0xFULL, 6)==1);
	BOOST_REQUIRE(bit_magic::find_open(0xFULL, 7)==0);

	for(uint64_t k=0; k<10000000; ++k ){//  random test cases
		uint64_t x = (((uint64_t)rand())<<32) + rand();
		for(int j=0;j<=64;++j){
			if( ((x>>j)&1)==0  ){// if there is a closing parenthesis at position j
				uint8_t res = bit_magic::find_open_naive(x,j);
				uint8_t res2 = bit_magic::find_open(x, j);
				if( res!=res2){
					BOOST_MESSAGE("x="<<x<<" j="<<j<<" res="<<(int)res<<" res2="<<(int)res2);
					BOOST_REQUIRE(res==res2);
				}
			}
		}
	}


}

BOOST_AUTO_TEST_SUITE_END()
