#define BOOST_TEST_DYN_LINK
#define BOOST_TEST_MAIN 
#include <boost/test/unit_test.hpp>
#include <boost/filesystem.hpp>
#include <sdsl/suffixarrays.hpp>
#include <sdsl/testutils.hpp>
#include <sdsl/algorithms.hpp>
#include <sdsl/lcp_sada.hpp>
#include <sdsl/coder.hpp>
#include <vector>
#include <string>
#include <iomanip>
//#include <algorithm> // for stl conform test 
//#include <fstream>
#if BOOST_WORKAROUND( __GNUC__, < 3 )
typedef boost::test_tools::output_test_stream onullstream_type;
#else
#include <boost/test/utils/nullstream.hpp>
typedef boost::onullstream onullstream_type;
#endif

using namespace boost::unit_test;
using namespace boost::filesystem;
using namespace sdsl;
using std::string;

const uint32_t sample_psi = 256;
const uint16_t sample_sa = 16;
typedef csa_sada_prac<enc_vector_prac<coder::fibonacci,sample_psi>, sample_sa> tCSAFib;
typedef tCSAFib::size_type size_type;

BOOST_AUTO_TEST_SUITE( csa_fib_test_suite )

BOOST_AUTO_TEST_CASE( constructor_test ){
	unit_test_log.set_threshold_level( log_messages );
//	BOOST_TEST_MESSAGE("argc = "<<boost::unit_test::framework::master_test_suite().argc);
//	BOOST_TEST_MESSAGE("argv[0] = "<<boost::unit_test::framework::master_test_suite().argv[0]);
	BOOST_TEST_MESSAGE( "Running tests for the construction of csa_sada_prac with fibonacci code..." );
	onullstream_type dev_null;
	path test_file_dir("@TESTCASESDIR@/small");
	for(directory_iterator dir_it(test_file_dir), end; dir_it != end; ++dir_it){
		if( is_regular( dir_it->status() ) ){
			if(file_size( *dir_it ) > 20){
				continue;
			}
			string test_case = dir_it->path().leaf();	
			BOOST_MESSAGE("test case: "<<test_case);
			char *text = NULL;
			size_type size = 0;
			if( (size = file::readText( dir_it->path().file_string().c_str(), text ) )>0 ){
				BOOST_MESSAGE("create csa");
				tCSAFib csa((const unsigned char*)text);
				csa_uncompressed ucsa((const unsigned char*)text);
				BOOST_CHECK( csa.size() == size+1 );
				BOOST_CHECK( ucsa.size() == size+1 );
				if(csa.size() < 20){
					lcp_sada<tCSAFib> lcp(&csa, text, csa); 
					lcp_sada<tCSAFib> lcp2 = lcp;
					lcp = lcp2;
					tCSAFib csa2((const unsigned char*)"abababa");
/*					{
						lcp_sada<tCSAFib> lcp3(&csa2, "abababa");
						lcp3.swap(lcp);
					}
*/					for(lcp_sada<tCSAFib>::const_iterator it=lcp.begin(), end = lcp.end(); it!=end; ++it){
						BOOST_MESSAGE(""<<std::setw(3)<<*it<<" "<<end-it);
					}
					for(size_type i=0; i<csa2.size(); ++i){
						BOOST_MESSAGE(""<<std::setw(3)<<i<<" "<<csa2[i]<<" "<<csa2.psi[i]<<" "/*<<lcp[i]*/<<" "<<" "<<algorithm::extract(csa2, csa2[i], csa2.size()));
					}
					BOOST_MESSAGE("text = "<<text);
					for(size_type i=0; i<csa.size(); ++i){
						BOOST_MESSAGE(""<<std::setw(3)<<i<<" "<<csa[i]<<" "<<csa.psi[i]<<" "/*<<lcp[i]*/<<" "<<lcp2[i]<<" "<<algorithm::extract(csa, csa[i], csa.size()));
					}
				}

				std::string text2 = algorithm::extract(csa, 0, csa.size());
				BOOST_MESSAGE("check if the text extracted from the csa equals the original text");
				for(size_type i=0; i<csa.size(); ++i){
					BOOST_CHECK( text[i] == text2[i] );
					if( text[i] != text2[i] ){
						BOOST_MESSAGE("index "<<i<<" '"<<text[i]<<"'!='"<<text2[i]<<"'");
					}
				}
				std::string text3 = algorithm::extract(ucsa, 0, ucsa.size());

				BOOST_MESSAGE("check if the text extracted from the csa (uncompressed) equals the original text");
				for(size_type i=0; i<csa.size(); ++i){
					BOOST_CHECK( text[i] == text3[i] );
					if( text[i] != text3[i] ){
						BOOST_MESSAGE("index "<<i<<" '"<<text[i]<<"'!='"<<text3[i]<<"'");
					}
				}

				BOOST_MESSAGE("check if the locate function works correct");
				srand(67);
				const size_type pattern_len = 6;
				size_type occ_begin=0, occ_end = 0, occs = 0;
				for(size_type i=0, idx=0; i < 100; ++i){
					idx = rand()%csa.size();
					if(idx + pattern_len > csa.size())
						continue;
					char temp = *(text+idx+pattern_len);
					*(text+idx+pattern_len) = '\0';
//					BOOST_MESSAGE("pattern = "<<(text+idx));
					*(text+idx+pattern_len) = temp;
					occs = algorithm::backward_search(csa, (unsigned char*)(text+idx), pattern_len, occ_begin, occ_end);
//					BOOST_MESSAGE("occurences "<<occs);
					// calc naive the occurenes of pattern in the text
					size_type occ_naive=0;
					for(size_type j=0;j<=csa.size()-pattern_len;++j){
						for(size_type k=0;k<pattern_len;++k)
							if( *(text+j+k) != *(text+idx+k) ){
								--occ_naive;
								break;
							}
						++occ_naive;
					}
					BOOST_CHECK( occs = occ_naive );
				}	
				delete [] text;
			}else{
				BOOST_MESSAGE("could not read test case");
			}
		}	
	}
}

BOOST_AUTO_TEST_SUITE_END()
