#define BOOST_TEST_DYN_LINK
#define BOOST_TEST_MAIN 
#include <boost/test/unit_test.hpp>
#include <sdsl/enc_vector_prac.hpp>
#include <sdsl/algorithms.hpp>
#include <algorithm> // for stl conform test 
#include <vector>

using namespace boost::unit_test;
using namespace sdsl;

BOOST_AUTO_TEST_SUITE( enc_vector_test_suite )

BOOST_AUTO_TEST_CASE( constructor_test ){
	unit_test_log.set_threshold_level( log_messages );
	BOOST_TEST_MESSAGE( "Running tests for the constructor of enc_vector_prac..." );
	// Test default constructor
	enc_vector_prac<> v;
	BOOST_CHECK_EQUAL( v.size(), (enc_vector_prac<>::size_type)0 );
	std::vector<size_t> a(10,0);
	for(size_t i=0;i<a.size();++i)
		a[i] = 2*i;
	v.init(a);
	BOOST_CHECK_EQUAL( v.size(), a.size() );
	for(size_t i=0; i<v.size(); ++i)
		if(v[i]!=a[i]){
			BOOST_TEST_MESSAGE( "v differs at position "<<i<<" from a ("<<v[i]<<"!="<<a[i]<<")" );
			BOOST_REQUIRE(false);
			break;
		}
	enc_vector_prac<> v2(a);
	BOOST_CHECK_EQUAL( v2.size(), a.size() );
	for(size_t i=0; i<v.size(); ++i)
		if(v[i]!=v2[i]){
			BOOST_TEST_MESSAGE( "v2 differs at position "<<i<<" from v ("<<v2[i]<<"!="<<v[i]<<")" );
			break;
		}
	BOOST_CHECK_EQUAL( v.size(), v2.size() );
	BOOST_CHECK( v == v2 );
	BOOST_CHECK( v2 == v2 );
	
	std::vector<size_t> b(10000,0);
	enc_vector_prac<> v3;
//	BOOST_CHECK_THROW( enc_vector_prac<>(b), std::logic::error );
	for(size_t i=0;i<b.size();++i)
		b[i] = b.size()-i;
	v3 = enc_vector_prac<>(b);
	BOOST_CHECK_EQUAL( v3.size(), b.size() );
	for(size_t i=0; i<v3.size(); ++i){
		if(v3[i]!=b[i]){
			BOOST_TEST_MESSAGE( "v3 differs at position "<<i<<" from b ("<<v3[i]<<"!="<<b[i]<<")" );
			BOOST_REQUIRE(false);
			break;
		}
	}

	BOOST_CHECK( v!=v3 );
}
/*
BOOST_AUTO_TEST_CASE( operator_test ){
	unit_test_log.set_threshold_level( log_messages );
	BOOST_TEST_MESSAGE( "Running tests for operators of enc_vector..." );
	std::vector<int> a(10);
	for(size_t i=0;i<a.size();++i)
		a[i] = i+1;
	enc_vector<> v1(a), v2, v3; 
	BOOST_CHECK( v2.empty() );
	BOOST_CHECK( v1!=v2 );
	v3 = v1;
	BOOST_CHECK(v3 == v1);
	v1.swap(v2);
	BOOST_CHECK( v1.empty() );
//	for(
//	enc_vector<SDSFibonacciCoder>::reverse_iterator it = v.rbegin();// it != v.rbegin(); ){
//	enc_vector<SDSFibonacciCoder>::reverse_iterator ite(v.end());
//	BOOST_CHECK( it == ite );
//		BOOST_MESSAGE(".");
//		BOOST_MESSAGE(".."<<*(it+2));
//		BOOST_MESSAGE("...");
//	}

//		BOOST_MESSAGE("end");
}

BOOST_AUTO_TEST_CASE( serialization_test ){
	BOOST_TEST_MESSAGE( "Running tests for serialization of enc_vector..." );
	std::vector<int> a(100000);
	a[0] = rand()%100;
	for(size_t i=1;i<a.size();++i)
		a[i] = a[i-1] + rand()%1000;

	const char * file_name = "enc_vector_serialize_test";

	enc_vector<> v1(a);
	BOOST_CHECK( v1.size() == a.size() );
	BOOST_CHECK( util::storeToFile( v1, file_name ) );

	enc_vector<> v2;
	BOOST_CHECK( util::loadFromFile( v2, file_name ) );
	BOOST_CHECK( algorithm::equal_container_values(v1, v2) );
	std::remove( file_name );
}

BOOST_AUTO_TEST_CASE( data_access_test )
{
}

BOOST_AUTO_TEST_CASE( stl_conform_test ){
	unit_test_log.set_threshold_level( log_messages );
	BOOST_TEST_MESSAGE( "Running tests for stl conformance of SDSBitVector..." );
}
*/
BOOST_AUTO_TEST_SUITE_END()

