#define BOOST_TEST_DYN_LINK
#define BOOST_TEST_MAIN 
#include <boost/test/unit_test.hpp>
#include <sdsl/nearest_neighbour_dictionary.hpp>
#include <sdsl/vectors.hpp>
#include <sdsl/util.hpp>

using namespace boost::unit_test;
using namespace sdsl;

typedef bit_vector::size_type size_type;

BOOST_AUTO_TEST_SUITE( nearest_neighbour_dictionary_test_suite )

BOOST_AUTO_TEST_CASE( constructor_test ){
	unit_test_log.set_threshold_level( log_messages );
	bit_vector v(1000035);

	BOOST_MESSAGE("size of v = "<<util::get_size_in_bytes(v));

	util::set_zero_bits(v);
	for(size_type i=0; i<v.size(); ++i){
		if(rand()%32==0){
			v[i] = 1;
		}
	}
	nearest_neighbour_dictionary<30> nnd(v);
	BOOST_MESSAGE("nnd size = "<<util::get_size_in_bytes(nnd));

	for(size_type i=0, ones = 0, last_one_pos=0; i<v.size(); ++i){
		if( v[i] ){
			++ones;
			size_type s = nnd.select(ones);	// test select method
			if(s != i){
				BOOST_MESSAGE("select("<<ones<<")="<<s<<"  should be "<<i );
			}
			BOOST_REQUIRE( nnd.select(ones) == i );
			size_type r = nnd.rank(i+1);  // test rank method
			if(r != ones){
				BOOST_MESSAGE("rank("<<i+1<<")="<<r<<"  should be "<<ones );
			}
			BOOST_REQUIRE( r == ones );
			last_one_pos = i;
		}
		if( ones > 0 ){ 
			size_type p = nnd.prev(i); // test prev method
			if(p != last_one_pos ){
				BOOST_MESSAGE("prev("<<i<<")="<<p<<" should be "<< last_one_pos );
			}
			BOOST_REQUIRE( p == last_one_pos );
		}
	}

	for(size_type i=v.size(), last_one_pos=v.size(), ones=0; i>0; --i){
		if(v[i]){
			++ones;
			last_one_pos = i;
		}
		if(ones>0){
			size_type n = nnd.next(i);
			if( n != last_one_pos ){
				BOOST_MESSAGE("next("<<i<<")="<<n<<" should be "<< last_one_pos );
			}
			BOOST_REQUIRE( n == last_one_pos );
		}
	}
}


BOOST_AUTO_TEST_SUITE_END()
