#define BOOST_TEST_DYN_LINK
#define BOOST_TEST_MAIN 
#include <boost/test/unit_test.hpp>
#include <sdsl/int_vector.hpp>
#include <sdsl/range_minimum_support_sparse_table.hpp>
//#include <sdsl/range_minimum_support.hpp>
#include <sdsl/util.hpp>
#include <vector>

typedef std::vector<uint64_t> 	tVI;
typedef std::vector<tVI>		tVVI;

using namespace boost::unit_test;
using namespace sdsl;

BOOST_AUTO_TEST_SUITE( range_minimum_support_sparse_table_test )

BOOST_AUTO_TEST_CASE( method_test ){
	typedef bit_vector::size_type size_type;
	unit_test_log.set_threshold_level( log_messages );
	BOOST_TEST_MESSAGE( "Running tests for methods of range_minimum_support_sparse_table" );
	const size_type test_cases = 3;
	int_vector<> v[test_cases];
	v[0] = int_vector<>(10);
	v[0][0] = 4; v[0][1] = 2; v[0][2] = 7; v[0][3] = 9; v[0][4] = 1;
	v[0][5] = 8; v[0][6] = 0; v[0][7] = 5; v[0][8] = 3; v[0][9] = 6;

	v[1] = int_vector<>(2000);
	for(size_type i=0; i< v[1].size(); ++i)
		v[1][i] = rand()%v[1].size();

	v[2] = int_vector<>(1000000);
	for(size_type i=0; i< v[2].size(); ++i)
		v[2][i] = rand()%v[2].size();

	for(size_type tc=0; tc<test_cases; ++tc ){
		BOOST_MESSAGE("create range minimum support for test case "<<tc+1);	
		typedef range_minimum_support_sparse_table<> rmq_ds;
		typedef range_maximum_support_sparse_table<>::type RMQ_ds;
		rmq_ds rmsst = rmq_ds(&v[tc]);
		BOOST_MESSAGE("range minimum support created ");	

		bool tested_serialize_and_load = false;
test_methods:		

		BOOST_MESSAGE("size in bytes: "<<util::get_size_in_bytes(rmsst));
		const size_type n = v[tc].size(); 
		if( n <= 2000 ){ // test all queries
			BOOST_MESSAGE("check naive all queries");
			tVVI rmq(n, tVI(n, 0));
			for(size_type i=0; i<n; ++i) rmq[i][i] = i;
			for(size_type i=0; i<n; ++i)
				for(size_type j=i+1;j<n;++j){
					rmq[i][j] = v[tc][rmq[i][j-1]] <= v[tc][j] ? rmq[i][j-1] : j;
				}
			for(size_type i=0; i<n; ++i)
				for(size_type j=i;j<n;++j){
					size_type r = rmsst(i,j);
					if( r != rmq[i][j] ){
						BOOST_MESSAGE("rmq("<<i<<","<<j<<")="<<r<<" expected="<<rmq[i][j]);
						BOOST_MESSAGE("v["<<r<<"]="<<v[tc][r]<<" v["<<rmq[i][j]<<"]="<<v[tc][rmq[i][j]]);
					}
					BOOST_REQUIRE( r == rmq[i][j] );
				}
		}else{ // test some queries
			BOOST_MESSAGE("check some queries");
			size_type q = 0;
			size_type steps = 10000*n;
			while(1){
				size_type i = rand()%n;
				size_type j = rand()%n;
				if( i > j ){ std::swap(i, j); }
				if( j-i+1 > steps )
					break;
				steps -= (j-i+1);
				size_type rmq = i;
				for(size_type k=i; k<=j; ++k)
					if( v[tc][k] < v[tc][rmq] )
						rmq = k;
				size_type r = rmsst(i, j);
				++q;
				if( r != rmq){
					BOOST_MESSAGE("rmq("<<i<<","<<j<<")="<<r<<" expected="<<rmq);
					BOOST_MESSAGE("v["<<r<<"]="<<v[tc][r]<<" v["<<rmq<<"]="<<v[tc][rmq]);
				}
				BOOST_REQUIRE( r == rmq );
			}
			BOOST_MESSAGE("checked "<<q<<" queries");
		}
		if( tested_serialize_and_load )
			continue;
		BOOST_MESSAGE("check serialize");
		std::string file_name = "/tmp/test_rmq";
		util::store_to_file(rmsst, file_name.c_str() );

		BOOST_MESSAGE("Check load");
		std::ifstream in;
		in.open(file_name.c_str(), std::ios::binary | std::ios::in );
		BOOST_REQUIRE( in );
		rmsst.load(in, &v[tc]);
		in.close();

		tested_serialize_and_load = true;
		goto test_methods; 
	}
}



BOOST_AUTO_TEST_SUITE_END()
