#define BOOST_TEST_DYN_LINK
#define BOOST_TEST_MAIN 
#include <boost/test/unit_test.hpp>
#include <sdsl/vectors.hpp>
#include <ctime> // for performance measurement

using namespace boost::unit_test;
using namespace sdsl;

BOOST_AUTO_TEST_SUITE( rank_support_test_suite )

BOOST_AUTO_TEST_CASE( constructor_test ){
	unit_test_log.set_threshold_level( log_messages );
	BOOST_TEST_MESSAGE( "Running tests for the constructor of rank_support_jmc and rank_support_v..." );
	int_vector<1> v(5800050);
	util::set_random_bits(v);
	// Test default constructor
	rank_support_jmc rs_v(&v);
	rank_support_v<>   rsv_v(&v);
	for(size_t i=0, sum=0, size = v.size(); i<size; ++i){
		if(v[i])
			++sum;
		BOOST_CHECK(rs_v.rank(i+1)==sum);
		BOOST_CHECK(rsv_v.rank(i+1)==sum);
	}

	int_vector<1> w(0);
	rank_support_jmc rs_w(&w);
	rank_support_v<> rsv_w(&w);
}

BOOST_AUTO_TEST_CASE( operator_test ){
	unit_test_log.set_threshold_level( log_messages );
	BOOST_TEST_MESSAGE( "Running tests for operators of rank_support_jmc and rank_support_v..." );
	int_vector<1> v(100000), w(100000);
	util::set_random_bits(v);
	util::set_random_bits(w);
	if(v[0]==w[0])
		v[0] = 1-w[0];
	
	rank_support_jmc rs_v(&v), rs_vc(&v);
	rank_support_v<> rsv_v(&v), rsv_vc(&v);
	// Test == operator
	BOOST_CHECK( rs_v == rs_vc );
	BOOST_CHECK( rsv_v == rsv_vc );

	rank_support_jmc rs_vcc;
	rank_support_v<> rsv_vcc;

	BOOST_CHECK( rs_vcc != rs_v );
	BOOST_CHECK( rsv_vcc != rsv_v );

	// Test assignment operator
	rs_vcc = rs_v;
	rsv_vcc = rsv_v;

	BOOST_CHECK( rs_vcc == rs_vc );
	BOOST_CHECK( rsv_vcc == rsv_vc );
	BOOST_CHECK( rs_vcc == rs_v );
	BOOST_CHECK( rsv_vcc == rsv_v );

	rank_support_jmc rs_w(&w);
	rank_support_v<> rsv_w(&w);

	BOOST_CHECK( rs_w != rs_v );
	BOOST_CHECK( rsv_w != rsv_v );

	rs_w.swap(rs_v);
	rsv_w.swap(rsv_v);

	BOOST_CHECK( rs_vc != rs_w );
	BOOST_CHECK( rsv_vc != rsv_w );
}


BOOST_AUTO_TEST_CASE( methods_test )
{
	unit_test_log.set_threshold_level( log_messages );
	BOOST_TEST_MESSAGE( "Running methods tests for rank_support_jmc and rank_support_v..." );
	size_t s = 2*64;
	int_vector<1> v(s);
	util::set_one_bits(v);

	rank_support_jmc rs(&v);
	rank_support_v<> rsv(&v);
	for(size_t i=0, size=v.size();i<size;++i){
		BOOST_CHECK( rs.rank(i+1) == i+1  );
		BOOST_CHECK( rsv.rank(i+1) == i+1  );
	}

	int_vector<1> w(50);
	util::set_random_bits(w);
	rank_support_jmc rs_w(&w);
	rank_support_jmc rsv_w(&w);
	for(size_t i=1,j=0, size=w.size(); i<=size;++i){
		j += w[i-1];
		BOOST_CHECK(rs_w.rank(i)==j);
		BOOST_CHECK(rsv_w.rank(i)==j);
	}
}


BOOST_AUTO_TEST_SUITE_END()
