#define BOOST_TEST_DYN_LINK
#define BOOST_TEST_MAIN 
#include <boost/test/unit_test.hpp>
#include <boost/random/linear_congruential.hpp>
#include <sdsl/rank_support_v.hpp>
#include <ctime> // for performance measurement

#include <iostream>

using namespace boost::unit_test;
using namespace sdsl;

BOOST_AUTO_TEST_SUITE( rank_support10_test_suite )

BOOST_AUTO_TEST_CASE ( vigna_10_test ){
	unit_test_log.set_threshold_level( log_messages );
	BOOST_TEST_MESSAGE ( "Running tests for Vigna 10-Occurencies..." );

	int_vector<1> v(1<<20);
	util::set_random_bits(v);

	rank_support_v<10,2>   rsv_v(0);
	rsv_v.init(&v);

	for(size_t i=1, sum=0, size = v.size(); i<size; ++i){
		//if (v[i] == 1 && v[i-1] == 0)
		if (v[i-1] == 1 && v[i] == 0)
			sum++;
		if (sum != rsv_v.rank(i+1))
			std:: cout << i << " ";
		BOOST_CHECK_EQUAL( rsv_v.rank(i+1), sum );
	}
}

BOOST_AUTO_TEST_CASE( operator_test ){
	unit_test_log.set_threshold_level( log_messages );
	BOOST_TEST_MESSAGE( "Running tests for operators of rank_support_v<10,2>..." );
	int_vector<1> v(100000), w(100000);
	util::set_random_bits(v);
	util::set_random_bits(w);
	if(v[0]==w[0])
		v[0] = 1-w[0];
	
	rank_support_v<10,2> rsv_v(&v), rsv_vc(&v);
	// Test == operator
	BOOST_CHECK( rsv_v == rsv_vc );

	rank_support_v<10,2> rsv_vcc;

	BOOST_CHECK( rsv_vcc != rsv_v );

	// Test assignment operator
	rsv_vcc = rsv_v;

	BOOST_CHECK( rsv_vcc == rsv_vc );
	BOOST_CHECK( rsv_vcc == rsv_v );

	rank_support_v<10,2> rsv_w(&w);

	BOOST_CHECK( rsv_w != rsv_v );

	rsv_w.swap(rsv_v);

	BOOST_CHECK( rsv_vc != rsv_w );
}



BOOST_AUTO_TEST_SUITE_END()
