#define BOOST_TEST_DYN_LINK
#define BOOST_TEST_MAIN 
#include <boost/test/unit_test.hpp>
#include <sdsl/vectors.hpp>

using namespace boost::unit_test;
using namespace sdsl;

BOOST_AUTO_TEST_SUITE( select_support_test_suite )

BOOST_AUTO_TEST_CASE( constructor_test ){
	unit_test_log.set_threshold_level( log_messages );
	BOOST_TEST_MESSAGE( "Running tests for the constructor of select_support..." );
	int_vector<1> v(10000);
	util::set_random_bits(v);
	// Test default constructor
	select_support_mcl<1> ss_v(&v);
	size_t i = 0, j = 0;
	for(int_vector<1>::const_iterator v_it = v.begin(); v_it != v.end(); ++v_it, ++i){
		if(*v_it){
			++j;
			if( ss_v.select(j) != i ){
				BOOST_MESSAGE("j = "<<j<<" ss_v.select(j)="<<ss_v.select(j)<<" i="<<i);
			}
			BOOST_CHECK( ss_v.select(j) == i );
		}
	}
}

BOOST_AUTO_TEST_CASE( operator_test ){
	unit_test_log.set_threshold_level( log_messages );
	BOOST_TEST_MESSAGE( "Running tests for operators of select_support..." );
	bit_vector v(1000000), w(1000000);
	util::set_random_bits(v);
	util::set_random_bits(w);
	v[0] = 1;
	if( v[0]==w[0] )	// make v and w different
		v[0] = 1-w[0];
	
	for(bit_vector::size_type i=1; i<160000; ++i)
		v[i] = 0;
	v[2] = 1;
	for(size_t i=840000; i<850000; ++i)
		v[i] = 1;
	bit_vector::size_type ones = util::get_one_bits(v), pos = v.size()-1;
	while( (ones%4096) != 0  ){
		ones -=  v[pos];
		--pos;
	}

	for(bit_vector::size_type i=pos-160000; i<pos;++i)
		v[i] = 0;


	BOOST_TEST_MESSAGE("Build select supports");
	select_support_mcl<1> ss_v(&v), ss_vc(&v);
	select_support_mcl<0> ss0_v(&v);
	select_support_mcl<10,2> ss10_v(&v);
	select_support_mcl<01,2> ss01_v(&v);
	BOOST_TEST_MESSAGE("Builded select supports");

	bit_vector::size_type i = 0, j = 0;
	for(bit_vector::const_iterator v_it = v.begin(); v_it != v.end(); ++v_it, ++i){
		if(*v_it){
			++j;
			if( ss_v.select(j) != i ){
				BOOST_MESSAGE("j = "<<j<<" ss_v.select(j)="<<ss_v.select(j)<<" i="<<i);
			}
			BOOST_REQUIRE( ss_v.select(j) == i );
		}
	}
	i = 0, j = 0;
	for(bit_vector::const_iterator v_it = v.begin(); v_it != v.end(); ++v_it, ++i){
		if(!(*v_it)){
			++j;
			if( ss0_v.select(j) != i ){
				BOOST_MESSAGE("j = "<<j<<" ss0_v.select(j)="<<ss0_v.select(j)<<" i="<<i);
			}
			BOOST_REQUIRE( ss0_v.select(j) == i );
		}
	}

	i = 0, j = 0;
	for(bit_vector::const_iterator v_it = v.begin(); v_it != v.end(); ++v_it, ++i){
		if( i > 0 and !(*v_it) and *(v_it-1) ){
			++j;
			if( ss10_v.select(j) != i ){
				BOOST_MESSAGE("j = "<<j<<" ss10_v.select(j)="<<ss10_v.select(j)<<" i="<<i);
			}
			BOOST_REQUIRE( ss10_v.select(j) == i );
		}
	}
	
	i = 0, j = 0;
	for(bit_vector::const_iterator v_it = v.begin(); v_it != v.end(); ++v_it, ++i){
		if( i > 0 and (*v_it) and !*(v_it-1) ){
			++j;
			bit_vector::size_type res = ss01_v.select(j);
			if( res != i ){
				BOOST_MESSAGE("j = "<<j<<" ss01_v.select(j)="<<res<<" i="<<i);
			}
			BOOST_REQUIRE( res == i );
		}
	}

	// Test == operator
	BOOST_CHECK( ss_v == ss_vc );

	select_support_mcl<1> ss_vcc;
	
	// Test != operator
	BOOST_CHECK( ss_v != ss_vcc );
	
	// Test assignment operator
	ss_vcc = ss_v;

	BOOST_CHECK( ss_vcc == ss_v );
	BOOST_CHECK( ss_vcc == ss_vc );

	select_support_mcl<1> ss_w(&w);

	BOOST_CHECK( ss_w != ss_v  );

	ss_w.swap(ss_v);

	BOOST_CHECK( ss_w != ss_vc );

	int_vector<1> uv(1000), uw(1000);
	util::set_random_bits(uv);
	uv[51] = uv[50];
	uw = uv;
	uw[50] = 1-uv[50];
	uv[51] = 1-uw[51];
	BOOST_CHECK(util::get_one_bits(uv)==util::get_one_bits(uw));
	BOOST_CHECK(uv!=uw);
	select_support_mcl<1> ss_uv(&uv), ss_uw(&uw);

	BOOST_CHECK( ss_uv != ss_uw );
}


BOOST_AUTO_TEST_CASE( methods_test )
{
	unit_test_log.set_threshold_level( log_messages );
	BOOST_TEST_MESSAGE( "Running methods tests for select_support_mcl..." );
	size_t s = 2*64;
	int_vector<1> v(s);
	util::set_one_bits(v);

	select_support_mcl<1> ss_v(&v);
	for(size_t i=1;i<=v.size();++i){
		BOOST_CHECK( ss_v.select(i) == i-1  );
	}

	int_vector<1> w(100);
	util::set_random_bits(w);
	for(size_t i=0; i<w.size()/3; ++i)
		w[i] = 0;
	select_support_mcl<1> ss_w1(&w);
	for(size_t i=1, j=0; i <= util::get_one_bits(w) ;++i, ++j){
		while(j < w.size())
			if( w[j] )
				break;
			else
				++j;
		BOOST_CHECK( ss_w1.select(i) == j );
	}
	select_support_mcl<0> ss_w0(&w);
	for(size_t i=1, j=0; i <= w.bit_size()-util::get_one_bits(w); ++i, ++j){
		while(j < w.size())
			if( w[j]==0 )
				break;
			else
				++j;
		BOOST_CHECK( ss_w0.select(i) == j );
	}
}

BOOST_AUTO_TEST_SUITE_END()

