#define BOOST_TEST_DYN_LINK
#define BOOST_TEST_MAIN 
#include <boost/test/unit_test.hpp>
#include <sdsl/vectors.hpp>
#include <sdsl/select_support.hpp>
#include <iostream>

using namespace boost::unit_test;
using namespace sdsl;

BOOST_AUTO_TEST_SUITE( select_support_test_suite )


BOOST_AUTO_TEST_CASE( select_test ){
	unit_test_log.set_threshold_level( log_messages );
	BOOST_TEST_MESSAGE( "Running tests for the select<1> and select<0> of select_support..." );
	int_vector<1> v(100);
	util::set_random_bits(v);
	// Test default constructor
	select_support_mcl<10,2> ss_v1(&v);
	select_support_mcl<01,2> ss_v0(&v);
	size_t  j = 0, k=0;
	for(bit_vector::size_type i=1; i < v.size();++i){
		if(!v[i] and v[i-1]){
			++j;
			BOOST_CHECK( ss_v1.select(j) == i );
		}
		else if(v[i] and !v[i-1]){
			++k;
			BOOST_CHECK( ss_v0.select(k) == i );
		}
	}
}


BOOST_AUTO_TEST_CASE ( select10_test ){
	unit_test_log.set_threshold_level( log_messages );
	BOOST_TEST_MESSAGE( "Running tests for 10-select_support..." );
	//int_vector<1> v((1<<30));
	bit_vector v((1<<27));
	util::set_random_bits(v);

	/* generate some long blocks */
	for (int i=1<<20; i<2*(1<<20); i++)
		v[i] = 0;
	for (int i=1<<20; i<2*(1<<20); i+=4096)
		v[i] = 1;
	for (int i=10*(1<<20); i<12*(1<<20); i++)
		v[i] = 0;
	for (int i=10*(1<<20); i<12*(1<<20); i+=4096)
		v[i] = 1;
	for (int i=15*(1<<20); i<40*(1<<20); i++)
		v[i] = 0;
	for (int i=15*(1<<20); i<40*(1<<20); i+=(1<<18))
		v[i] = 1;


/*
	std:: cout << "tested vector:\n";	
	for (int i=0; i<(int)v.size(); i++){
		if (i % 8   == 0 && i) std:: cout << " ";
		if (i % 64  == 0 && i) std:: cout << std:: endl;
		if (i % 512 == 0 && i) std:: cout << std:: endl;
		std:: cout << v[i]; 
	} std:: cout << std:: endl;
*/

	select_support_mcl<10,2> ss_v;
	ss_v.init(&v);
	size_t i = 0, j = 0;
	bool found1 = 0;
	for(int_vector<1>::const_iterator v_it = v.begin(); v_it != v.end(); ++v_it, ++i){
		if (*v_it == 0 && found1){
			j++;
			if ( ss_v.select(j) != i ){
				std::cerr << "Found " << j << " occurencies until position " << i << ", but select10 tells us position " << ss_v.select(j) << " for the " << j << "th occurence of 10.\n";
				assert(0);
			}
 			BOOST_CHECK_EQUAL ( ss_v.select(j) , i );
		} 
		found1 = 0;
		if(*v_it == 1)
			found1 = 1;
	}

}

BOOST_AUTO_TEST_CASE ( select01_test ){
	unit_test_log.set_threshold_level( log_messages );
	BOOST_TEST_MESSAGE( "Running tests for 01-select_support..." );
	//int_vector<1> v((1<<30));
	int_vector<1> v( (1<<27));
	util::set_random_bits(v);

	/* generate some long blocks */
	
	for (int i=1<<20; i<2*(1<<20); i++)
		v[i] = 0;
	for (int i=1<<20; i<2*(1<<20); i+=4096)
		v[i] = 1;
	for (int i=10*(1<<20); i<12*(1<<20); i++)
		v[i] = 0;
	for (int i=10*(1<<20); i<12*(1<<20); i+=4096)
		v[i] = 1;
	for (int i=15*(1<<20); i<50*(1<<20); i++)
		v[i] = 0;
	for (int i=15*(1<<20); i<50*(1<<20); i+=(1<<18))
		v[i] = 1;

/*
	std:: cout << "tested vector:\n";	
	for (int i=0; i<(int)v.size(); i++){
		if (i % 8   == 0 && i) std:: cout << " ";
		if (i % 64  == 0 && i) std:: cout << std:: endl;
		if (i % 512 == 0 && i) std:: cout << std:: endl;
		std:: cout << v[i]; 
	} std:: cout << std:: endl;
*/

	select_support_mcl<10,2> ss_v;
	ss_v.init(&v);
	size_t i = 0, j = 0;
	bool found0 = 0;
	for(int_vector<1>::const_iterator v_it = v.begin(); v_it != v.end(); ++v_it, ++i){
		if (*v_it == 1 && found0){
			j++;
			uint64_t sel = ss_v.select(j);
			if ( sel != i ){
				std::cerr << "Found " << j << " occurencies until position " << i << ", but select10 tells us position " << sel << " for the " << j << "th occurence of 10.\n";
				assert(0);
			}
 			BOOST_CHECK_EQUAL ( sel, i );
		} 
		found0 = 0;
		if(*v_it == 0)
			found0 = 1;
	}

}



BOOST_AUTO_TEST_CASE( operator_test ){
	unit_test_log.set_threshold_level( log_messages );
	BOOST_TEST_MESSAGE( "Running tests for operators of select_support..." );
	int_vector<1> v(100000), w(100000);
	util::set_random_bits(v);
	util::set_random_bits(w);
	v[0] = 1;
	if( v[0]==w[0] )	// make v and w different
		v[0] = 1-w[0];
	
	for(size_t i=1; i<84000; ++i)
		v[i] = 0;
	for(size_t i=84000; i<85000; ++i)
		v[i] = 1;

	select_support_mcl<10,2> ss_v(&v), ss_vc(&v);
	// Test == operator
	BOOST_CHECK( ss_v == ss_vc );

	select_support_mcl<10,2> ss_vcc;
	
	// Test != operator
	BOOST_CHECK( ss_v != ss_vcc );
	
	// Test assignment operator
	ss_vcc = ss_v;

	BOOST_CHECK( ss_vcc == ss_v );
	BOOST_CHECK( ss_vcc == ss_vc );

	select_support_mcl<10,2> ss_w(&w);

	BOOST_CHECK( ss_w != ss_v  );

	ss_w.swap(ss_v);

	BOOST_CHECK( ss_w != ss_vc );

	int_vector<1> uv(1000), uw(1000);
	util::set_random_bits(uv);
	uv[51] = uv[50];
	uw = uv;
	uw[50] = 1-uv[50];
	uv[51] = 1-uw[51];
	BOOST_CHECK(util::get_one_bits(uv)==util::get_one_bits(uw));
	BOOST_CHECK(uv!=uw);
	select_support_mcl<10,2> ss_uv(&uv), ss_uw(&uw);

	BOOST_CHECK( ss_uv != ss_uw );
}

BOOST_AUTO_TEST_SUITE_END()

