#define BOOST_TEST_DYN_LINK
#define BOOST_TEST_MAIN 
#include <boost/test/unit_test.hpp>
#include <sdsl/suffixarrays.hpp>
#include <vector>
//#include <algorithm> // for stl conform test 
//#include <fstream>
#if BOOST_WORKAROUND( __GNUC__, < 3 )
typedef boost::test_tools::output_test_stream onullstream_type;
#else
#include <boost/test/utils/nullstream.hpp>
typedef boost::onullstream onullstream_type;
#endif

using namespace boost::unit_test;
using namespace sdsl;

BOOST_AUTO_TEST_SUITE( SDSCompressedSuffixArray_test_suite )

BOOST_AUTO_TEST_CASE( compressed_suffix_array_constructor_test ){
	unit_test_log.set_threshold_level( log_messages );
	BOOST_TEST_MESSAGE( "Running tests for the construction of csa_sada_prac and csa_sada_theo..." );
	onullstream_type dev_null;
	// First test 4 simple texts
	const unsigned char *text1 = (unsigned char*)"swissmissismissing";
	const unsigned char *text2 = (unsigned char*)"umulmundumulmundumulmundumulmherum";
	unsigned char	   *text3 = new unsigned char[10001];
	unsigned char 	   *text4 = new unsigned char[1000001];
	size_t len1 = strlen((char*)text1),
		   len2 = strlen((char*)text2),
		   len3 = 10000,
		   len4 = 1000000;

	for(size_t i=0;i<len3; ++i) text3[i] = 'a';
	text3[len3] = '\0';
	for(size_t i=0;i<len4; ++i) text4[i] = 'a'+rand()%26;
	text4[len4] = '\0';

	csa_sada_theo<> csa;
	BOOST_CHECK( csa.size() == 0 );
	BOOST_CHECK( csa.empty() );
	BOOST_MESSAGE("check constructor of csa_sada_theo with const char* argument");
	csa = csa_sada_theo<>(text1);
	BOOST_MESSAGE("csa for text1 constructed");
	BOOST_CHECK( !csa.empty() );
	BOOST_CHECK( algorithm::verify_sa(text1, len1+1, csa) );
	csa = csa_sada_theo<>(text2);
	BOOST_MESSAGE("csa for text2 constructed");
	BOOST_CHECK( algorithm::verify_sa(text2, len2+1, csa) );
	csa = csa_sada_theo<>((const unsigned char*)text3);
	BOOST_MESSAGE("csa for text3 constructed");
	BOOST_CHECK( algorithm::verify_sa(text3, len3+1, csa) );
	csa = csa_sada_theo<>((const unsigned char*)text4);
	BOOST_MESSAGE("csa for text4 constructed");
	BOOST_CHECK( algorithm::verify_sa(text4, len4+1, csa) );

	BOOST_MESSAGE("check constructor of csa_sada_theo with SA as argument");
	std::vector<uint8_t> sa5(0);
	csa = csa_sada_theo<>(sa5,(const unsigned char*)"");
	BOOST_MESSAGE("csa for sa5 constructed");
	BOOST_CHECK(csa.empty());
	BOOST_CHECK( algorithm::equal_container_values(sa5, csa) );
	std::vector<uint8_t> sa6(5);
	sa6[0]=4; sa6[1]=2; sa6[2]=3; sa6[3]=0; sa6[4]=1;
	csa = csa_sada_theo<>(sa6,(const unsigned char*)"ecdab");
	BOOST_MESSAGE("csa for sa6 constructed");
	BOOST_CHECK( algorithm::equal_container_values(sa6, csa) );
	std::vector<uint32_t> sa7(len3);
	algorithm::calculate_sa( text3, len3, sa7 );
	csa = csa_sada_theo<>(sa7, text3);
	BOOST_MESSAGE("csa for sa7 constructed");
	BOOST_CHECK( algorithm::equal_container_values(sa7, csa) );
	std::vector<uint32_t> sa8(len4);
	algorithm::calculate_sa( text4, len4, sa8 );
	csa = csa_sada_theo<>(sa8, text4);
	BOOST_MESSAGE("csa for sa8 constructed");
	BOOST_CHECK( algorithm::equal_container_values(sa8, csa) );

	BOOST_MESSAGE("size of csa "<< csa.serialize(dev_null) );
	BOOST_MESSAGE("size of sa8 "<< sa8.size()*sizeof(sa8[0]));

//	BOOST_MESSAGE("Construct csa_sada_prac from sa8.");
//	csa_sada_prac<> csasl_test((const csa_sada_theo<>)csa);
//	BOOST_CHECK( algorithm::equal_container_values(csa, csasl_test) );

	std::vector<uint32_t> psi8;
	algorithm::sa2psi(sa8, psi8);
	enc_vector<> psi(psi8);
	BOOST_MESSAGE("size of psi "<<psi.serialize(dev_null));


	csa_sada_prac<> csasl;
	BOOST_CHECK( csasl.size() == 0 );
	BOOST_CHECK( csasl.empty() );
	BOOST_MESSAGE("check constructor of csa_sada_theo with const char* argument");
	csasl = csa_sada_prac<>(text1);
	BOOST_MESSAGE("csasl for text1 constructed");
	BOOST_CHECK( !csasl.empty() );
	BOOST_CHECK( algorithm::verify_sa(text1, len1+1, csasl) );
	csasl = csa_sada_prac<>(text2);
	BOOST_MESSAGE("csasl for text2 constructed");
	BOOST_CHECK( algorithm::verify_sa(text2, len2+1, csasl) );
	BOOST_MESSAGE("csasl for text2 verified");
	csasl = csa_sada_prac<>((const unsigned char*)text3);
	BOOST_MESSAGE("csasl for text3 constructed");
	BOOST_CHECK( algorithm::verify_sa(text3, len3+1, csasl) );
	csasl = csa_sada_prac<>((const unsigned char*)text4);
	BOOST_MESSAGE("csasl for text4 constructed");
	BOOST_CHECK( algorithm::verify_sa(text4, len4+1, csasl) );

//	BOOST_MESSAGE("check constructor of csa_sada_prac with SA as argument");
//	csasl = csa_sada_prac<>(sa5);
//	BOOST_MESSAGE("csasl for sa5 constructed");
//	BOOST_CHECK( algorithm::equal_container_values(sa5, csasl) );
//	csasl = csa_sada_prac<>(sa6);
//	BOOST_MESSAGE("csasl for sa6 constructed");
//	BOOST_CHECK( algorithm::equal_container_values(sa6, csasl) );
//	csasl = csa_sada_prac<>(sa7);
//	BOOST_MESSAGE("csasl for sa7 constructed");
//	BOOST_CHECK( algorithm::equal_container_values(sa7, csasl) );
//	csasl = csa_sada_prac<>(sa8);
//	BOOST_MESSAGE("csa for sa8 constructed");
//	BOOST_CHECK( algorithm::equal_container_values(sa8, csasl) );

//	BOOST_MESSAGE("size of csasl "<< csasl.serialize(dev_null) );
//	BOOST_MESSAGE("size of sa8 "<< sa8.size()*sizeof(sa8[0]));

//	BOOST_MESSAGE("size of psi "<<psi.serialize(dev_null));
}

BOOST_AUTO_TEST_CASE( compressed_suffix_array_method_test ){
	csa_sada_theo<> csa1((const unsigned char*)"hallodrio"), csa2((const unsigned char*)"holladiho"), csa3((const unsigned char*)"hallodrio");
	BOOST_CHECK( algorithm::equal_container_values(csa1, csa3) );
	BOOST_MESSAGE("check swap method");
	csa1.swap(csa2);
	BOOST_CHECK( algorithm::equal_container_values(csa2, csa3) );
	BOOST_MESSAGE("check begin() and end()");
	csa_sada_theo<> csa4;
	BOOST_CHECK( csa4.empty() );
	BOOST_CHECK( csa4.size() == 0 );
	BOOST_CHECK( csa4.begin() == csa4.end() );
	csa4 = csa_sada_theo<>((const unsigned char*)"swissmissismissing");
	BOOST_CHECK( csa4.begin() != csa4.end()  );
	BOOST_CHECK( csa4.begin() + csa4.size() == csa4.end() );
	BOOST_CHECK( csa4.end() - csa4.begin() == (long long)csa4.size() );

	csa_sada_prac<> csasl1((const unsigned char*)"hallodrio"), csasl2((const unsigned char*)"holladiho"), csasl3((const unsigned char*)"hallodrio");
	BOOST_CHECK( algorithm::equal_container_values(csasl1, csasl3) );
	BOOST_MESSAGE("check swap method");
	csasl1.swap(csasl2);
	BOOST_CHECK( algorithm::equal_container_values(csasl2, csasl3) );
	BOOST_MESSAGE("check begin() and end()");
	csa_sada_prac<> csasl4;
	BOOST_CHECK( csasl4.empty() );
	BOOST_CHECK( csasl4.size() == 0 );
	BOOST_CHECK( csasl4.begin() == csasl4.end() );
	csasl4 = csa_sada_prac<>((const unsigned char*)"swissmissismissing");
	BOOST_CHECK( csasl4.begin() != csasl4.end()  );
	BOOST_CHECK( csasl4.begin() + csasl4.size() == csasl4.end() );
	BOOST_CHECK( csasl4.end() - csasl4.begin() == (long long)csasl4.size() );
	
}

BOOST_AUTO_TEST_CASE( compressed_suffix_array_serialize_test ){
	onullstream_type dev_null;

	BOOST_MESSAGE("check serialize and load");
	size_t len = 1000000;
	unsigned char   *text = new unsigned char[1000001];
	for(size_t i=0;i<len; ++i) text[i] = 'a'+rand()%26;
	text[len] = '\0';

	const char * file_name = "csa_serialize_test";	

	csa_sada_theo<> csa((const unsigned char*)text);
	BOOST_CHECK( util::store_to_file(csa, file_name) );

	csa_sada_theo<> csa2;
	BOOST_CHECK( csa2.empty() );

	BOOST_CHECK( util::load_from_file(csa2, file_name) );
	
	BOOST_CHECK( algorithm::equal_container_values(csa, csa2) );

	csa = csa_sada_theo<>();

	BOOST_CHECK( util::store_to_file(csa, file_name) );
	BOOST_CHECK( util::load_from_file(csa2, file_name) );
	BOOST_CHECK( algorithm::equal_container_values(csa, csa2) );
	
	BOOST_MESSAGE("check serialize and load");
	csa_sada_prac<> csasl((const unsigned char*)text);
	BOOST_CHECK( util::store_to_file(csasl, file_name) );

	csa_sada_prac<> csasl2;
	BOOST_CHECK( csasl2.empty() );

	BOOST_CHECK( util::load_from_file(csasl2, file_name) );

	BOOST_CHECK( algorithm::equal_container_values(csasl, csasl2) );

//	csasl = csa_sada_theo<>();

//	BOOST_CHECK( util::store_to_file(csasl, file_name) );
//	BOOST_CHECK( util::load_from_file(csasl2, file_name) );
//	BOOST_CHECK( algorithm::equal_container_values(csasl, csasl2) );
	std::remove( file_name );
	
}

BOOST_AUTO_TEST_SUITE_END()
